@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 #include "LogBackend.hxx"
 #include "Log.hxx"
 #include "util/Domain.hxx"
-#include "util/StringUtil.hxx"
+#include "util/StringStrip.hxx"
 
 #include <assert.h>
 #include <stdio.h>
@@ -38,7 +38,7 @@
 #include "Main.hxx"
 
 static int
-ToAndroidLogLevel(LogLevel log_level)
+ToAndroidLogLevel(LogLevel log_level) noexcept
 {
 	switch (log_level) {
 	case LogLevel::DEBUG:
@@ -70,13 +70,13 @@ static bool enable_syslog;
 #endif
 
 void
-SetLogThreshold(LogLevel _threshold)
+SetLogThreshold(LogLevel _threshold) noexcept
 {
 	log_threshold = _threshold;
 }
 
 void
-EnableLogTimestamp()
+EnableLogTimestamp() noexcept
 {
 #ifdef HAVE_SYSLOG
 	assert(!enable_syslog);
@@ -86,7 +86,8 @@ EnableLogTimestamp()
 	enable_timestamp = true;
 }
 
-static const char *log_date(void)
+static const char *
+log_date() noexcept
 {
 	static constexpr size_t LOG_DATE_BUF_SIZE = 16;
 	static char buf[LOG_DATE_BUF_SIZE];
@@ -100,7 +101,7 @@ static const char *log_date(void)
  * characters.
  */
 static int
-chomp_length(const char *p)
+chomp_length(const char *p) noexcept
 {
 	size_t length = strlen(p);
 	return StripRight(p, length);
@@ -108,8 +109,9 @@ chomp_length(const char *p)
 
 #ifdef HAVE_SYSLOG
 
+gcc_const
 static int
-ToSysLogLevel(LogLevel log_level)
+ToSysLogLevel(LogLevel log_level) noexcept
 {
 	switch (log_level) {
 	case LogLevel::DEBUG:
@@ -133,7 +135,7 @@ ToSysLogLevel(LogLevel log_level)
 }
 
 static void
-SysLog(const Domain &domain, LogLevel log_level, const char *message)
+SysLog(const Domain &domain, LogLevel log_level, const char *message) noexcept
 {
 	syslog(ToSysLogLevel(log_level), "%s: %.*s",
 	       domain.GetName(),
@@ -141,14 +143,14 @@ SysLog(const Domain &domain, LogLevel log_level, const char *message)
 }
 
 void
-LogInitSysLog()
+LogInitSysLog() noexcept
 {
 	openlog(PACKAGE, 0, LOG_DAEMON);
 	enable_syslog = true;
 }
 
 void
-LogFinishSysLog()
+LogFinishSysLog() noexcept
 {
 	if (enable_syslog)
 		closelog();
@@ -157,7 +159,7 @@ LogFinishSysLog()
 #endif
 
 static void
-FileLog(const Domain &domain, const char *message)
+FileLog(const Domain &domain, const char *message) noexcept
 {
 	fprintf(stderr, "%s%s: %.*s\n",
 		enable_timestamp ? log_date() : "",
@@ -174,7 +176,7 @@ FileLog(const Domain &domain, const char *message)
 #endif /* !ANDROID */
 
 void
-Log(const Domain &domain, LogLevel level, const char *msg)
+Log(const Domain &domain, LogLevel level, const char *msg) noexcept
 {
 #ifdef ANDROID
 	__android_log_print(ToAndroidLogLevel(level), "MPD",
