@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 #include "client/Response.hxx"
 
 void
-tag_print_types(Response &r)
+tag_print_types(Response &r) noexcept
 {
 	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; i++)
 		if (IsTagEnabled(i))
@@ -32,20 +32,22 @@ tag_print_types(Response &r)
 }
 
 void
-tag_print(Response &r, TagType type, const char *value)
+tag_print(Response &r, TagType type, const char *value) noexcept
 {
 	r.Format("%s: %s\n", tag_item_names[type], value);
 }
 
 void
-tag_print_values(Response &r, const Tag &tag)
+tag_print_values(Response &r, const Tag &tag) noexcept
 {
+	const auto tag_mask = r.GetTagMask();
 	for (const auto &i : tag)
-		r.Format("%s: %s\n", tag_item_names[i.type], i.value);
+		if (tag_mask.Test(i.type))
+			tag_print(r, i.type, i.value);
 }
 
 void
-tag_print(Response &r, const Tag &tag)
+tag_print(Response &r, const Tag &tag) noexcept
 {
 	if (!tag.duration.IsNegative())
 		r.Format("Time: %i\n"
