@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,7 +36,7 @@ Context::GetCacheDir(JNIEnv *env) const
 	jobject file = env->CallObjectMethod(Get(), method);
 	if (file == nullptr) {
 		env->ExceptionClear();
-		return AllocatedPath::Null();
+		return nullptr;
 	}
 
 	return Java::File::ToAbsolutePath(env, file);
