@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,11 +19,12 @@
 
 #include "config.h"
 #include "ArchivePlugin.hxx"
+#include "ArchiveFile.hxx"
 #include "fs/Path.hxx"
 
 #include <assert.h>
 
-ArchiveFile *
+std::unique_ptr<ArchiveFile>
 archive_file_open(const ArchivePlugin *plugin, Path path)
 {
 	assert(plugin != nullptr);
