@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,12 +26,16 @@
 #include "db/Count.hxx"
 #include "db/Selection.hxx"
 #include "CommandError.hxx"
+#include "protocol/RangeArg.hxx"
 #include "client/Client.hxx"
 #include "client/Response.hxx"
-#include "tag/Tag.hxx"
+#include "tag/ParseName.hxx"
+#include "tag/Mask.hxx"
 #include "util/ConstBuffer.hxx"
+#include "util/Exception.hxx"
 #include "util/StringAPI.hxx"
-#include "SongFilter.hxx"
+#include "util/ASCII.hxx"
+#include "song/Filter.hxx"
 #include "BulkEdit.hxx"
 
 #include <memory>
@@ -40,7 +44,7 @@ CommandResult
 handle_listfiles_db(Client &client, Response &r, const char *uri)
 {
 	const DatabaseSelection selection(uri, false);
-	db_selection_print(r, client.partition,
+	db_selection_print(r, client.GetPartition(),
 			   selection, false, true);
 	return CommandResult::OK;
 }
@@ -49,34 +53,67 @@ CommandResult
 handle_lsinfo2(Client &client, const char *uri, Response &r)
 {
 	const DatabaseSelection selection(uri, false);
-	db_selection_print(r, client.partition,
+	db_selection_print(r, client.GetPartition(),
 			   selection, true, false);
 	return CommandResult::OK;
 }
 
+static TagType
+ParseSortTag(const char *s)
+{
+	if (StringIsEqualIgnoreCase(s, "Last-Modified"))
+		return TagType(SORT_TAG_LAST_MODIFIED);
+
+	TagType tag = tag_name_parse_i(s);
+	if (tag == TAG_NUM_OF_ITEM_TYPES)
+		throw ProtocolError(ACK_ERROR_ARG, "Unknown sort tag");
+
+	return tag;
+}
+
 static CommandResult
 handle_match(Client &client, Request args, Response &r, bool fold_case)
 {
-	RangeArg window;
+	RangeArg window = RangeArg::All();
 	if (args.size >= 2 && StringIsEqual(args[args.size - 2], "window")) {
 		window = args.ParseRange(args.size - 1);
 
 		args.pop_back();
 		args.pop_back();
-	} else
-		window.SetAll();
-
-	SongFilter filter;
-	if (!filter.Parse(args, fold_case)) {
-		r.Error(ACK_ERROR_ARG, "incorrect arguments");
-		return CommandResult::ERROR;
 	}
 
-	const DatabaseSelection selection("", true, &filter);
+	TagType sort = TAG_NUM_OF_ITEM_TYPES;
+	bool descending = false;
+	if (args.size >= 2 && StringIsEqual(args[args.size - 2], "sort")) {
+		const char *s = args.back();
+		if (*s == '-') {
+			descending = true;
+			++s;
+		}
 
-	db_selection_print(r, client.partition,
-			   selection, true, false,
-			   window.start, window.end);
+		sort = ParseSortTag(s);
+
+		args.pop_back();
+		args.pop_back();
+	}
+
+	SongFilter filter;
+	try {
+		filter.Parse(args, fold_case);
+	} catch (...) {
+		r.Error(ACK_ERROR_ARG,
+			GetFullMessage(std::current_exception()).c_str());
+		return CommandResult::ERROR;
+	}
+	filter.Optimize();
+
+	DatabaseSelection selection("", true, &filter);
+	selection.window = window;
+	selection.sort = sort;
+	selection.descending = descending;
+
+	db_selection_print(r, client.GetPartition(),
+			   selection, true, false);
 	return CommandResult::OK;
 }
 
@@ -96,15 +133,20 @@ static CommandResult
 handle_match_add(Client &client, Request args, Response &r, bool fold_case)
 {
 	SongFilter filter;
-	if (!filter.Parse(args, fold_case)) {
-		r.Error(ACK_ERROR_ARG, "incorrect arguments");
+	try {
+		filter.Parse(args, fold_case);
+	} catch (...) {
+		r.Error(ACK_ERROR_ARG,
+			GetFullMessage(std::current_exception()).c_str());
 		return CommandResult::ERROR;
 	}
+	filter.Optimize();
 
-	const ScopeBulkEdit bulk_edit(client.partition);
+	auto &partition = client.GetPartition();
+	const ScopeBulkEdit bulk_edit(partition);
 
 	const DatabaseSelection selection("", true, &filter);
-	AddFromDatabase(client.partition, selection);
+	AddFromDatabase(partition, selection);
 	return CommandResult::OK;
 }
 
@@ -126,14 +168,18 @@ handle_searchaddpl(Client &client, Request args, Response &r)
 	const char *playlist = args.shift();
 
 	SongFilter filter;
-	if (!filter.Parse(args, true)) {
-		r.Error(ACK_ERROR_ARG, "incorrect arguments");
+	try {
+		filter.Parse(args, true);
+	} catch (...) {
+		r.Error(ACK_ERROR_ARG,
+			GetFullMessage(std::current_exception()).c_str());
 		return CommandResult::ERROR;
 	}
+	filter.Optimize();
 
 	const Database &db = client.GetDatabaseOrThrow();
 
-	search_add_to_playlist(db, *client.GetStorage(),
+	search_add_to_playlist(db, client.GetStorage(),
 			       "", playlist, &filter);
 	return CommandResult::OK;
 }
@@ -156,12 +202,19 @@ handle_count(Client &client, Request args, Response &r)
 	}
 
 	SongFilter filter;
-	if (!args.IsEmpty() && !filter.Parse(args, false)) {
-		r.Error(ACK_ERROR_ARG, "incorrect arguments");
-		return CommandResult::ERROR;
+	if (!args.empty()) {
+		try {
+			filter.Parse(args, false);
+		} catch (...) {
+			r.Error(ACK_ERROR_ARG,
+				GetFullMessage(std::current_exception()).c_str());
+			return CommandResult::ERROR;
+		}
+
+		filter.Optimize();
 	}
 
-	PrintSongCount(r, client.partition, "", &filter, group);
+	PrintSongCount(r, client.GetPartition(), "", &filter, group);
 	return CommandResult::OK;
 }
 
@@ -171,20 +224,43 @@ handle_listall(Client &client, Request args, Response &r)
 	/* default is root directory */
 	const auto uri = args.GetOptional(0, "");
 
-	db_selection_print(r, client.partition,
+	db_selection_print(r, client.GetPartition(),
 			   DatabaseSelection(uri, true),
 			   false, false);
 	return CommandResult::OK;
 }
 
+static CommandResult
+handle_list_file(Client &client, Request args, Response &r)
+{
+	std::unique_ptr<SongFilter> filter;
+
+	if (!args.empty()) {
+		filter.reset(new SongFilter());
+		try {
+			filter->Parse(args, false);
+		} catch (...) {
+			r.Error(ACK_ERROR_ARG,
+				GetFullMessage(std::current_exception()).c_str());
+			return CommandResult::ERROR;
+		}
+		filter->Optimize();
+	}
+
+	PrintSongUris(r, client.GetPartition(), filter.get());
+	return CommandResult::OK;
+}
+
 CommandResult
 handle_list(Client &client, Request args, Response &r)
 {
 	const char *tag_name = args.shift();
-	unsigned tagType = locate_parse_type(tag_name);
+	if (StringEqualsCaseASCII(tag_name, "file") ||
+	    StringEqualsCaseASCII(tag_name, "filename"))
+		return handle_list_file(client, args, r);
 
-	if (tagType >= TAG_NUM_OF_ITEM_TYPES &&
-	    tagType != LOCATE_TAG_FILE_TYPE) {
+	const auto tagType = tag_name_parse_i(tag_name);
+	if (tagType == TAG_NUM_OF_ITEM_TYPES) {
 		r.FormatError(ACK_ERROR_ARG,
 			      "Unknown tag type: %s", tag_name);
 		return CommandResult::ERROR;
@@ -202,7 +278,7 @@ handle_list(Client &client, Request args, Response &r)
 			return CommandResult::ERROR;
 		}
 
-		filter.reset(new SongFilter((unsigned)TAG_ARTIST,
+		filter.reset(new SongFilter(TAG_ARTIST,
 					    args.shift()));
 	}
 
@@ -220,12 +296,16 @@ handle_list(Client &client, Request args, Response &r)
 		args.pop_back();
 	}
 
-	if (!args.IsEmpty()) {
+	if (!args.empty()) {
 		filter.reset(new SongFilter());
-		if (!filter->Parse(args, false)) {
-			r.Error(ACK_ERROR_ARG, "not able to parse args");
+		try {
+			filter->Parse(args, false);
+		} catch (...) {
+			r.Error(ACK_ERROR_ARG,
+				GetFullMessage(std::current_exception()).c_str());
 			return CommandResult::ERROR;
 		}
+		filter->Optimize();
 	}
 
 	if (tagType < TAG_NUM_OF_ITEM_TYPES && tagType == group) {
@@ -233,7 +313,7 @@ handle_list(Client &client, Request args, Response &r)
 		return CommandResult::ERROR;
 	}
 
-	PrintUniqueTags(r, client.partition,
+	PrintUniqueTags(r, client.GetPartition(),
 			tagType, group, filter.get());
 	return CommandResult::OK;
 }
@@ -244,7 +324,7 @@ handle_listallinfo(Client &client, Request args, Response &r)
 	/* default is root directory */
 	const auto uri = args.GetOptional(0, "");
 
-	db_selection_print(r, client.partition,
+	db_selection_print(r, client.GetPartition(),
 			   DatabaseSelection(uri, true),
 			   true, false);
 	return CommandResult::OK;
