@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,20 +19,12 @@
 
 #include "config.h"
 #include "Param.hxx"
-#include "ConfigPath.hxx"
+#include "Path.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "util/RuntimeError.hxx"
 
 #include <stdexcept>
 
-ConfigParam::ConfigParam(const char *_value, int _line)
-	:next(nullptr), value(_value), line(_line), used(false) {}
-
-ConfigParam::~ConfigParam()
-{
-	delete next;
-}
-
 AllocatedPath
 ConfigParam::GetPath() const
 {
