@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,10 @@
 #include "Interface.hxx"
 #include "Partition.hxx"
 #include "client/Response.hxx"
-#include "LightSong.hxx"
+#include "song/LightSong.hxx"
 #include "tag/Tag.hxx"
 #include "tag/VisitFallback.hxx"
+#include "TagPrint.hxx"
 
 #include <functional>
 #include <map>
@@ -42,7 +43,7 @@ class TagCountMap : public std::map<std::string, SearchStats> {
 };
 
 static void
-PrintSearchStats(Response &r, const SearchStats &stats)
+PrintSearchStats(Response &r, const SearchStats &stats) noexcept
 {
 	unsigned total_duration_s =
 		std::chrono::duration_cast<std::chrono::seconds>(stats.total_duration).count();
@@ -53,18 +54,18 @@ PrintSearchStats(Response &r, const SearchStats &stats)
 }
 
 static void
-Print(Response &r, TagType group, const TagCountMap &m)
+Print(Response &r, TagType group, const TagCountMap &m) noexcept
 {
 	assert(unsigned(group) < TAG_NUM_OF_ITEM_TYPES);
 
 	for (const auto &i : m) {
-		r.Format("%s: %s\n", tag_item_names[group], i.first.c_str());
+		tag_print(r, group, i.first.c_str());
 		PrintSearchStats(r, i.second);
 	}
 }
 
 static void
-stats_visitor_song(SearchStats &stats, const LightSong &song)
+stats_visitor_song(SearchStats &stats, const LightSong &song) noexcept
 {
 	stats.n_songs++;
 
@@ -85,11 +86,10 @@ CollectGroupCounts(TagCountMap &map, const Tag &tag,
 }
 
 static void
-GroupCountVisitor(TagCountMap &map, TagType group, const LightSong &song)
+GroupCountVisitor(TagCountMap &map, TagType group,
+		  const LightSong &song) noexcept
 {
-	assert(song.tag != nullptr);
-
-	const Tag &tag = *song.tag;
+	const Tag &tag = song.tag;
 	VisitTagWithFallbackOrEmpty(tag, group,
 				    std::bind(CollectGroupCounts, std::ref(map),
 					      std::cref(tag),
