@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,8 @@
 #include "lib/upnp/Util.hxx"
 #include "lib/expat/ExpatParser.hxx"
 #include "Tags.hxx"
-#include "tag/TagBuilder.hxx"
-#include "tag/TagTable.hxx"
+#include "tag/Builder.hxx"
+#include "tag/Table.hxx"
 #include "util/NumberParser.hxx"
 #include "util/StringView.hxx"
 
@@ -41,9 +41,9 @@ gcc_pure
 static UPnPDirObject::ItemClass
 ParseItemClass(StringView name) noexcept
 {
-	if (name.EqualsLiteral("object.item.audioItem.musicTrack"))
+	if (name.Equals("object.item.audioItem.musicTrack"))
 		return UPnPDirObject::ItemClass::MUSIC;
-	else if (name.EqualsLiteral("object.item.playlistItem"))
+	else if (name.Equals("object.item.playlistItem"))
 		return UPnPDirObject::ItemClass::PLAYLIST;
 	else
 		return UPnPDirObject::ItemClass::UNKNOWN;
@@ -55,23 +55,21 @@ ParseDuration(const char *duration) noexcept
 {
 	char *endptr;
 
-	unsigned result = ParseUnsigned(duration, &endptr);
+	int hours = ParseInt(duration, &endptr);
 	if (endptr == duration || *endptr != ':')
 		return SignedSongTime::Negative();
 
-	result *= 60;
 	duration = endptr + 1;
-	result += ParseUnsigned(duration, &endptr);
+	unsigned minutes = ParseUnsigned(duration, &endptr);
 	if (endptr == duration || *endptr != ':')
 		return SignedSongTime::Negative();
 
-	result *= 60;
 	duration = endptr + 1;
-	result += ParseUnsigned(duration, &endptr);
-	if (endptr == duration || *endptr != 0)
+	double seconds = ParseDouble(duration, &endptr);
+	if (endptr == duration || *endptr != 0 || seconds < 0.0)
 		return SignedSongTime::Negative();
 
-	return SignedSongTime::FromS(result);
+	return SignedSongTime::FromS((((hours * 60) + minutes) * 60) + seconds);
 }
 
 /**
