@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,14 +27,16 @@
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "input/TextInputStream.hxx"
-#include "util/StringUtil.hxx"
+#include "util/StringStrip.hxx"
 #include "Log.hxx"
 
-#include <stdexcept>
+#include <exception>
 
 #include <assert.h>
 #include <string.h>
 
+#ifdef HAVE_CLASS_GLOB
+
 inline void
 ExcludeList::ParseLine(char *line) noexcept
 {
@@ -43,6 +45,8 @@ ExcludeList::ParseLine(char *line) noexcept
 		patterns.emplace_front(p);
 }
 
+#endif
+
 bool
 ExcludeList::Load(InputStreamPtr is)
 {
@@ -54,7 +58,7 @@ ExcludeList::Load(InputStreamPtr is)
 		ParseLine(line);
 #else
 	/* not implemented */
-	(void)path_fs;
+	(void)is;
 #endif
 
 	return true;
