@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,17 +23,19 @@
 #include "../DecoderBuffer.hxx"
 #include "input/InputStream.hxx"
 #include "CheckAudioFormat.hxx"
-#include "tag/TagHandler.hxx"
+#include "tag/Handler.hxx"
+#include "util/ScopeExit.hxx"
 #include "util/ConstBuffer.hxx"
-#include "util/Error.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
 #include <neaacdec.h>
 
+#include <cmath>
+#include <exception>
+
 #include <assert.h>
 #include <string.h>
-#include <unistd.h>
 
 static const unsigned adts_sample_rates[] =
     { 96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050,
@@ -126,7 +128,7 @@ adts_song_duration(DecoderBuffer &buffer)
 
 		if (frames == 0) {
 			auto data = ConstBuffer<uint8_t>::FromVoid(buffer.Read());
-			assert(!data.IsEmpty());
+			assert(!data.empty());
 			assert(frame_length <= data.size);
 
 			sample_rate = adts_sample_rates[(data.data[2] & 0x3c) >> 2];
@@ -193,7 +195,10 @@ faad_song_duration(DecoderBuffer &buffer, InputStream &is)
 
 		auto song_length = adts_song_duration(buffer);
 
-		is.LockSeek(tagsize, IgnoreError());
+		try {
+			is.LockSeek(tagsize);
+		} catch (...) {
+		}
 
 		buffer.Clear();
 
@@ -243,16 +248,16 @@ faad_decoder_new()
 /**
  * Wrapper for NeAACDecInit() which works around some API
  * inconsistencies in libfaad.
+ *
+ * Throws #std::runtime_error on error.
  */
-static bool
+static void
 faad_decoder_init(NeAACDecHandle decoder, DecoderBuffer &buffer,
-		  AudioFormat &audio_format, Error &error)
+		  AudioFormat &audio_format)
 {
 	auto data = ConstBuffer<uint8_t>::FromVoid(buffer.Read());
-	if (data.IsEmpty()) {
-		error.Set(faad_decoder_domain, "Empty file");
-		return false;
-	}
+	if (data.empty())
+		throw std::runtime_error("Empty file");
 
 	uint8_t channels;
 	unsigned long sample_rate;
@@ -261,15 +266,13 @@ faad_decoder_init(NeAACDecHandle decoder, DecoderBuffer &buffer,
 				   const_cast<unsigned char *>(data.data),
 				   data.size,
 				   &sample_rate, &channels);
-	if (nbytes < 0) {
-		error.Set(faad_decoder_domain, "Not an AAC stream");
-		return false;
-	}
+	if (nbytes < 0)
+		throw std::runtime_error("Not an AAC stream");
 
 	buffer.Consume(nbytes);
 
-	return audio_format_init_checked(audio_format, sample_rate,
-					 SampleFormat::S16, channels, error);
+	audio_format = CheckAudioFormat(sample_rate, SampleFormat::S16,
+					channels);
 }
 
 /**
@@ -281,7 +284,7 @@ faad_decoder_decode(NeAACDecHandle decoder, DecoderBuffer &buffer,
 		    NeAACDecFrameInfo *frame_info)
 {
 	auto data = ConstBuffer<uint8_t>::FromVoid(buffer.Read());
-	if (data.IsEmpty())
+	if (data.empty())
 		return nullptr;
 
 	return NeAACDecDecode(decoder, frame_info,
@@ -306,22 +309,23 @@ faad_get_file_time(InputStream &is)
 
 	if (!recognized) {
 		NeAACDecHandle decoder = faad_decoder_new();
+		AtScopeExit(decoder) { NeAACDecClose(decoder); };
 
 		buffer.Fill();
 
 		AudioFormat audio_format;
-		if (faad_decoder_init(decoder, buffer, audio_format,
-				      IgnoreError()))
+		try {
+			faad_decoder_init(decoder, buffer, audio_format);
 			recognized = true;
-
-		NeAACDecClose(decoder);
+		} catch (...) {
+		}
 	}
 
 	return std::make_pair(recognized, duration);
 }
 
 static void
-faad_stream_decode(Decoder &mpd_decoder, InputStream &is,
+faad_stream_decode(DecoderClient &client, InputStream &is,
 		   DecoderBuffer &buffer, const NeAACDecHandle decoder)
 {
 	const auto total_time = faad_song_duration(buffer, is);
@@ -331,16 +335,12 @@ faad_stream_decode(Decoder &mpd_decoder, InputStream &is,
 
 	/* initialize it */
 
-	Error error;
 	AudioFormat audio_format;
-	if (!faad_decoder_init(decoder, buffer, audio_format, error)) {
-		LogError(error);
-		return;
-	}
+	faad_decoder_init(decoder, buffer, audio_format);
 
 	/* initialize the MPD core */
 
-	decoder_initialized(mpd_decoder, audio_format, false, total_time);
+	client.Ready(audio_format, false, total_time);
 
 	/* the decoder loop */
 
@@ -387,47 +387,42 @@ faad_stream_decode(Decoder &mpd_decoder, InputStream &is,
 		/* update bit rate and position */
 
 		if (frame_info.samples > 0) {
-			bit_rate = frame_info.bytesconsumed * 8.0 *
-			    frame_info.channels * audio_format.sample_rate /
-			    frame_info.samples / 1000 + 0.5;
+			bit_rate = lround(frame_info.bytesconsumed * 8.0 *
+					  frame_info.channels * audio_format.sample_rate /
+					  frame_info.samples / 1000);
 		}
 
 		/* send PCM samples to MPD */
 
-		cmd = decoder_data(mpd_decoder, is, decoded,
-				   (size_t)frame_info.samples * 2,
-				   bit_rate);
+		cmd = client.SubmitData(is, decoded,
+					(size_t)frame_info.samples * 2,
+					bit_rate);
 	} while (cmd != DecoderCommand::STOP);
 }
 
 static void
-faad_stream_decode(Decoder &mpd_decoder, InputStream &is)
+faad_stream_decode(DecoderClient &client, InputStream &is)
 {
-	DecoderBuffer buffer(&mpd_decoder, is,
+	DecoderBuffer buffer(&client, is,
 			     FAAD_MIN_STREAMSIZE * MAX_CHANNELS);
 
 	/* create the libfaad decoder */
 
 	const NeAACDecHandle decoder = faad_decoder_new();
+	AtScopeExit(decoder) { NeAACDecClose(decoder); };
 
-	faad_stream_decode(mpd_decoder, is, buffer, decoder);
-
-	/* cleanup */
-
-	NeAACDecClose(decoder);
+	faad_stream_decode(client, is, buffer, decoder);
 }
 
 static bool
-faad_scan_stream(InputStream &is,
-		 const struct tag_handler *handler, void *handler_ctx)
+faad_scan_stream(InputStream &is, TagHandler &handler) noexcept
 {
 	auto result = faad_get_file_time(is);
 	if (!result.first)
 		return false;
 
 	if (!result.second.IsNegative())
-		tag_handler_invoke_duration(handler, handler_ctx,
-					    SongTime(result.second));
+		handler.OnDuration(SongTime(result.second));
 	return true;
 }
 
