@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2016 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,8 @@
 #include "FfmpegIo.hxx"
 #include "../DecoderAPI.hxx"
 #include "input/InputStream.hxx"
-#include "util/Error.hxx"
+
+#include <exception>
 
 AvioStream::~AvioStream()
 {
@@ -34,49 +35,62 @@ AvioStream::~AvioStream()
 	}
 }
 
-static int
-mpd_ffmpeg_stream_read(void *opaque, uint8_t *buf, int size)
+inline int
+AvioStream::Read(void *dest, int size)
 {
-	AvioStream *stream = (AvioStream *)opaque;
-
-	return decoder_read(stream->decoder, stream->input,
-			    (void *)buf, size);
+	return decoder_read(client, input, dest, size);
 }
 
-static int64_t
-mpd_ffmpeg_stream_seek(void *opaque, int64_t pos, int whence)
+inline int64_t
+AvioStream::Seek(int64_t pos, int whence)
 {
-	AvioStream *stream = (AvioStream *)opaque;
-
 	switch (whence) {
 	case SEEK_SET:
 		break;
 
 	case SEEK_CUR:
-		pos += stream->input.GetOffset();
+		pos += input.GetOffset();
 		break;
 
 	case SEEK_END:
-		if (!stream->input.KnownSize())
+		if (!input.KnownSize())
 			return -1;
 
-		pos += stream->input.GetSize();
+		pos += input.GetSize();
 		break;
 
 	case AVSEEK_SIZE:
-		if (!stream->input.KnownSize())
+		if (!input.KnownSize())
 			return -1;
 
-		return stream->input.GetSize();
+		return input.GetSize();
 
 	default:
 		return -1;
 	}
 
-	if (!stream->input.LockSeek(pos, IgnoreError()))
+	try {
+		input.LockSeek(pos);
+		return input.GetOffset();
+	} catch (...) {
 		return -1;
+	}
+}
 
-	return stream->input.GetOffset();
+int
+AvioStream::_Read(void *opaque, uint8_t *buf, int size)
+{
+	AvioStream &stream = *(AvioStream *)opaque;
+
+	return stream.Read(buf, size);
+}
+
+int64_t
+AvioStream::_Seek(void *opaque, int64_t pos, int whence)
+{
+	AvioStream &stream = *(AvioStream *)opaque;
+
+	return stream.Seek(pos, whence);
 }
 
 bool
@@ -89,9 +103,8 @@ AvioStream::Open()
 
 	io = avio_alloc_context(buffer, BUFFER_SIZE,
 				false, this,
-				mpd_ffmpeg_stream_read, nullptr,
-				input.IsSeekable()
-				? mpd_ffmpeg_stream_seek : nullptr);
+				_Read, nullptr,
+				input.IsSeekable() ? _Seek : nullptr);
 	/* If avio_alloc_context() fails, who frees the buffer?  The
 	   libavformat API documentation does not specify this, it
 	   only says that AVIOContext.buffer must be freed in the end,
