@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,25 +27,26 @@
 #include <poll.h>
 #endif
 
-MultiSocketMonitor::MultiSocketMonitor(EventLoop &_loop)
-	:IdleMonitor(_loop), TimeoutMonitor(_loop) {
+MultiSocketMonitor::MultiSocketMonitor(EventLoop &_loop) noexcept
+	:IdleMonitor(_loop),
+	 timeout_event(_loop, BIND_THIS_METHOD(OnTimeout)) {
 }
 
 void
-MultiSocketMonitor::Reset()
+MultiSocketMonitor::Reset() noexcept
 {
-	assert(GetEventLoop().IsInsideOrNull());
+	assert(GetEventLoop().IsInside());
 
 	fds.clear();
 	IdleMonitor::Cancel();
-	TimeoutMonitor::Cancel();
+	timeout_event.Cancel();
 	ready = refresh = false;
 }
 
 void
-MultiSocketMonitor::ClearSocketList()
+MultiSocketMonitor::ClearSocketList() noexcept
 {
-	assert(GetEventLoop().IsInsideOrNull());
+	assert(GetEventLoop().IsInside());
 
 	fds.clear();
 }
@@ -53,13 +54,13 @@ MultiSocketMonitor::ClearSocketList()
 #ifndef _WIN32
 
 void
-MultiSocketMonitor::ReplaceSocketList(pollfd *pfds, unsigned n)
+MultiSocketMonitor::ReplaceSocketList(pollfd *pfds, unsigned n) noexcept
 {
 	pollfd *const end = pfds + n;
 
-	UpdateSocketList([pfds, end](int fd) -> unsigned {
+	UpdateSocketList([pfds, end](SocketDescriptor fd) -> unsigned {
 			auto i = std::find_if(pfds, end, [fd](const struct pollfd &pfd){
-					return pfd.fd == fd;
+					return pfd.fd == fd.Get();
 				});
 			if (i == end)
 				return 0;
@@ -71,24 +72,24 @@ MultiSocketMonitor::ReplaceSocketList(pollfd *pfds, unsigned n)
 
 	for (auto i = pfds; i != end; ++i)
 		if (i->events != 0)
-			AddSocket(i->fd, i->events);
+			AddSocket(SocketDescriptor(i->fd), i->events);
 }
 
 #endif
 
 void
-MultiSocketMonitor::Prepare()
+MultiSocketMonitor::Prepare() noexcept
 {
 	const auto timeout = PrepareSockets();
 	if (timeout >= timeout.zero())
-		TimeoutMonitor::Schedule(timeout);
+		timeout_event.Schedule(timeout);
 	else
-		TimeoutMonitor::Cancel();
+		timeout_event.Cancel();
 
 }
 
 void
-MultiSocketMonitor::OnIdle()
+MultiSocketMonitor::OnIdle() noexcept
 {
 	if (ready) {
 		ready = false;
