@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,8 @@
 
 #include "config.h"
 #include "Observer.hxx"
-#include "FilterInternal.hxx"
+#include "Filter.hxx"
+#include "Prepared.hxx"
 #include "util/ConstBuffer.hxx"
 
 #include <assert.h>
@@ -27,21 +28,20 @@
 class FilterObserver::PreparedProxy final : public PreparedFilter {
 	FilterObserver &observer;
 
-	PreparedFilter *const prepared_filter;
+	std::unique_ptr<PreparedFilter> prepared_filter;
 	Proxy *child = nullptr;
 
 public:
 	PreparedProxy(FilterObserver &_observer,
-		      PreparedFilter *_prepared_filter)
+		      std::unique_ptr<PreparedFilter> _prepared_filter)
 		:observer(_observer),
-		 prepared_filter(_prepared_filter) {}
+		 prepared_filter(std::move(_prepared_filter)) {}
 
 	~PreparedProxy() {
 		assert(child == nullptr);
 		assert(observer.proxy == this);
 
 		observer.proxy = nullptr;
-		delete prepared_filter;
 	}
 
 	void Clear(gcc_unused Proxy *_child) {
@@ -51,35 +51,38 @@ public:
 
 	Filter *Get();
 
-	Filter *Open(AudioFormat &af) override;
+	std::unique_ptr<Filter> Open(AudioFormat &af) override;
 };
 
 class FilterObserver::Proxy final : public Filter {
 	PreparedProxy &parent;
 
-	Filter *const filter;
+	std::unique_ptr<Filter> filter;
 
 public:
-	Proxy(PreparedProxy &_parent, Filter *_filter)
+	Proxy(PreparedProxy &_parent, std::unique_ptr<Filter> _filter)
 		:Filter(_filter->GetOutAudioFormat()),
-		 parent(_parent), filter(_filter) {}
+		 parent(_parent), filter(std::move(_filter)) {}
 
 	~Proxy() {
 		parent.Clear(this);
-		delete filter;
 	}
 
 	Filter *Get() {
-		return filter;
+		return filter.get();
 	}
 
-	void Reset() override {
+	void Reset() noexcept override {
 		filter->Reset();
 	}
 
 	ConstBuffer<void> FilterPCM(ConstBuffer<void> src) override {
 		return filter->FilterPCM(src);
 	}
+
+	ConstBuffer<void> Flush() override {
+		return filter->Flush();
+	}
 };
 
 Filter *
@@ -90,21 +93,24 @@ FilterObserver::PreparedProxy::Get()
 		: nullptr;
 }
 
-Filter *
+std::unique_ptr<Filter>
 FilterObserver::PreparedProxy::Open(AudioFormat &af)
 {
 	assert(child == nullptr);
 
-	Filter *f = prepared_filter->Open(af);
-	return child = new Proxy(*this, f);
+	auto c = std::make_unique<Proxy>(*this, prepared_filter->Open(af));
+	child = c.get();
+	return c;
 }
 
-PreparedFilter *
-FilterObserver::Set(PreparedFilter *pf)
+std::unique_ptr<PreparedFilter>
+FilterObserver::Set(std::unique_ptr<PreparedFilter> pf)
 {
 	assert(proxy == nullptr);
 
-	return proxy = new PreparedProxy(*this, pf);
+	auto p = std::make_unique<PreparedProxy>(*this, std::move(pf));
+	proxy = p.get();
+	return p;
 }
 
 Filter *
