@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,10 +26,11 @@
 
 #include "config.h"
 #include "filter/FilterPlugin.hxx"
-#include "filter/FilterInternal.hxx"
+#include "filter/Filter.hxx"
+#include "filter/Prepared.hxx"
 #include "filter/FilterRegistry.hxx"
 #include "AudioFormat.hxx"
-#include "Compiler.h"
+#include "util/Compiler.h"
 #include "util/ConstBuffer.hxx"
 
 class NullFilter final : public Filter {
@@ -43,15 +44,15 @@ public:
 
 class PreparedNullFilter final : public PreparedFilter {
 public:
-	virtual Filter *Open(AudioFormat &af) override {
-		return new NullFilter(af);
+	virtual std::unique_ptr<Filter> Open(AudioFormat &af) override {
+		return std::make_unique<NullFilter>(af);
 	}
 };
 
-static PreparedFilter *
+static std::unique_ptr<PreparedFilter>
 null_filter_init(gcc_unused const ConfigBlock &block)
 {
-	return new PreparedNullFilter();
+	return std::make_unique<PreparedNullFilter>();
 }
 
 const FilterPlugin null_filter_plugin = {
