@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,71 +19,61 @@
 
 #include "config.h"
 #include "VolumeFilterPlugin.hxx"
-#include "filter/FilterPlugin.hxx"
-#include "filter/FilterInternal.hxx"
-#include "filter/FilterRegistry.hxx"
+#include "filter/Filter.hxx"
+#include "filter/Prepared.hxx"
 #include "pcm/Volume.hxx"
 #include "AudioFormat.hxx"
 #include "util/ConstBuffer.hxx"
-#include "util/Error.hxx"
 
-#include <assert.h>
-#include <string.h>
+#include <stdexcept>
 
 class VolumeFilter final : public Filter {
 	PcmVolume pv;
 
 public:
-	unsigned GetVolume() const {
+	explicit VolumeFilter(const AudioFormat &audio_format)
+		:Filter(audio_format) {
+		pv.Open(out_audio_format.format);
+	}
+
+	unsigned GetVolume() const noexcept {
 		return pv.GetVolume();
 	}
 
-	void SetVolume(unsigned _volume) {
+	void SetVolume(unsigned _volume) noexcept {
 		pv.SetVolume(_volume);
 	}
 
 	/* virtual methods from class Filter */
-	AudioFormat Open(AudioFormat &af, Error &error) override;
-	void Close() override;
-	ConstBuffer<void> FilterPCM(ConstBuffer<void> src,
-				    Error &error) override;
+	ConstBuffer<void> FilterPCM(ConstBuffer<void> src) override;
 };
 
-static Filter *
-volume_filter_init(gcc_unused const config_param &param,
-		   gcc_unused Error &error)
-{
-	return new VolumeFilter();
-}
+class PreparedVolumeFilter final : public PreparedFilter {
+public:
+	/* virtual methods from class Filter */
+	std::unique_ptr<Filter> Open(AudioFormat &af) override;
+};
 
-AudioFormat
-VolumeFilter::Open(AudioFormat &audio_format, Error &error)
+std::unique_ptr<Filter>
+PreparedVolumeFilter::Open(AudioFormat &audio_format)
 {
-	if (!pv.Open(audio_format.format, error))
-		return AudioFormat::Undefined();
-
-	return audio_format;
-}
-
-void
-VolumeFilter::Close()
-{
-	pv.Close();
+	return std::make_unique<VolumeFilter>(audio_format);
 }
 
 ConstBuffer<void>
-VolumeFilter::FilterPCM(ConstBuffer<void> src, gcc_unused Error &error)
+VolumeFilter::FilterPCM(ConstBuffer<void> src)
 {
 	return pv.Apply(src);
 }
 
-const struct filter_plugin volume_filter_plugin = {
-	"volume",
-	volume_filter_init,
-};
+std::unique_ptr<PreparedFilter>
+volume_filter_prepare() noexcept
+{
+	return std::make_unique<PreparedVolumeFilter>();
+}
 
 unsigned
-volume_filter_get(const Filter *_filter)
+volume_filter_get(const Filter *_filter) noexcept
 {
 	const VolumeFilter *filter =
 		(const VolumeFilter *)_filter;
@@ -92,7 +82,7 @@ volume_filter_get(const Filter *_filter)
 }
 
 void
-volume_filter_set(Filter *_filter, unsigned volume)
+volume_filter_set(Filter *_filter, unsigned volume) noexcept
 {
 	VolumeFilter *filter = (VolumeFilter *)_filter;
 
