@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,15 +20,17 @@
 #include "config.h"
 #include "Config.hxx"
 #include "Charset.hxx"
-#include "config/ConfigGlobal.hxx"
+#include "config/Data.hxx"
 
 void
-ConfigureFS()
+ConfigureFS(const ConfigData &config)
 {
 #ifdef HAVE_FS_CHARSET
-	const char *charset = config_get_string(ConfigOption::FS_CHARSET);
+	const char *charset = config.GetString(ConfigOption::FS_CHARSET);
 	if (charset != nullptr)
 		SetFSCharset(charset);
+#else
+	(void)config;
 #endif
 }
 
