@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,15 +45,15 @@ ReadLink(Path path)
 #ifdef _WIN32
 	(void)path;
 	errno = EINVAL;
-	return AllocatedPath::Null();
+	return nullptr;
 #else
 	char buffer[MPD_PATH_MAX];
 	ssize_t size = readlink(path.c_str(), buffer, MPD_PATH_MAX);
 	if (size < 0)
-		return AllocatedPath::Null();
+		return nullptr;
 	if (size_t(size) >= MPD_PATH_MAX) {
 		errno = ENOMEM;
-		return AllocatedPath::Null();
+		return nullptr;
 	}
 	buffer[size] = '\0';
 	return AllocatedPath::FromFS(buffer);
@@ -72,11 +72,9 @@ TruncateFile(Path path)
 
 	CloseHandle(h);
 #else
-	int fd = open_cloexec(path.c_str(), O_WRONLY|O_TRUNC, 0);
-	if (fd < 0)
+	UniqueFileDescriptor fd;
+	if (!fd.Open(path.c_str(), O_WRONLY|O_TRUNC))
 		throw FormatErrno("Failed to truncate %s", path.c_str());
-
-	close(fd);
 #endif
 }
 
