@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,28 @@
 #include "Charset.hxx"
 
 std::string
-Path::ToUTF8() const
+Path::ToUTF8() const noexcept
+{
+	try {
+		return ToUTF8Throw();
+	} catch (...) {
+		return std::string();
+	}
+}
+
+std::string
+Path::ToUTF8Throw() const
 {
 	return ::PathToUTF8(c_str());
 }
+
+Path::const_pointer_type
+Path::GetSuffix() const noexcept
+{
+	const auto base = GetBase().c_str();
+	const auto *dot = StringFindLast(base, '.');
+	if (dot == nullptr || dot == base)
+		return nullptr;
+
+	return dot + 1;
+}
