@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,13 @@
 #include "AllocatedPath.hxx"
 
 AllocatedPath
-Path::GetDirectoryName() const
+Path::GetDirectoryName() const noexcept
 {
 	return AllocatedPath::FromFS(PathTraitsFS::GetParent(c_str()));
 }
+
+AllocatedPath
+operator/(Path a, Path b) noexcept
+{
+	return AllocatedPath::Build(a, b);
+}
