@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,18 +18,13 @@
  */
 
 #include "config.h"
-
-// Use X Desktop guidelines where applicable
-#if !defined(__APPLE__) && !defined(WIN32) && !defined(ANDROID)
-#define USE_XDG
-#endif
-
 #include "StandardDirectory.hxx"
 #include "FileSystem.hxx"
+#include "XDG.hxx"
 
 #include <array>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #include <shlobj.h>
 #else
@@ -39,7 +34,7 @@
 #endif
 
 #ifdef USE_XDG
-#include "util/StringUtil.hxx"
+#include "util/StringStrip.hxx"
 #include "util/StringCompare.hxx"
 #include "io/TextFile.hxx"
 #include <string.h>
@@ -53,7 +48,7 @@
 #include "Main.hxx"
 #endif
 
-#if !defined(WIN32) && !defined(ANDROID)
+#if !defined(_WIN32) && !defined(ANDROID)
 class PasswdEntry
 {
 #if defined(HAVE_GETPWNAM_R) || defined(HAVE_GETPWUID_R)
@@ -109,18 +104,18 @@ SafePathFromFS(PathTraitsFS::const_pointer_type dir)
 {
 	if (IsValidPathString(dir) && IsValidDir(dir))
 		return AllocatedPath::FromFS(dir);
-	return AllocatedPath::Null();
+	return nullptr;
 }
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 static AllocatedPath GetStandardDir(int folder_id)
 {
 	std::array<PathTraitsFS::value_type, MAX_PATH> dir;
 	auto ret = SHGetFolderPath(nullptr, folder_id | CSIDL_FLAG_DONT_VERIFY,
 				   nullptr, SHGFP_TYPE_CURRENT, dir.data());
 	if (FAILED(ret))
-		return AllocatedPath::Null();
+		return nullptr;
 	return SafePathFromFS(dir.data());
 }
 #endif
@@ -183,16 +178,16 @@ ParseConfigLine(char *line, const char *dir_name, AllocatedPath &result_dir)
 	*line_end = 0;
 
 	// build the result path
-	const char *path = line;
+	const auto path_fs = Path::FromFS(line);
 
-	auto result = AllocatedPath::Null();
+	AllocatedPath result = nullptr;
 	if (home_relative) {
 		auto home = GetHomeDir();
 		if (home.IsNull())
 			return true;
-		result = AllocatedPath::Build(home, path);
+		result = home / path_fs;
 	} else {
-		result = AllocatedPath::FromFS(path);
+		result = AllocatedPath(path_fs);
 	}
 
 	if (IsValidDir(result.c_str())) {
@@ -205,20 +200,19 @@ ParseConfigLine(char *line, const char *dir_name, AllocatedPath &result_dir)
 static AllocatedPath
 GetUserDir(const char *name) noexcept
 try {
-	auto result = AllocatedPath::Null();
+	AllocatedPath result = nullptr;
 	auto config_dir = GetUserConfigDir();
 	if (config_dir.IsNull())
 		return result;
-	auto dirs_file = AllocatedPath::Build(config_dir, "user-dirs.dirs");
 
-	TextFile input(dirs_file);
+	TextFile input(config_dir / Path::FromFS("user-dirs.dirs"));
 	char *line;
 	while ((line = input.ReadLine()) != nullptr)
 		if (ParseConfigLine(line, name, result))
 			return result;
 	return result;
 } catch (const std::exception &e) {
-	return AllocatedPath::Null();
+	return nullptr;
 }
 
 #endif
@@ -226,7 +220,7 @@ try {
 AllocatedPath
 GetUserConfigDir() noexcept
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	return GetStandardDir(CSIDL_LOCAL_APPDATA);
 #elif defined(USE_XDG)
 	// Check for $XDG_CONFIG_HOME
@@ -237,28 +231,28 @@ GetUserConfigDir() noexcept
 	// Check for $HOME/.config
 	auto home = GetHomeDir();
 	if (!home.IsNull()) {
-		AllocatedPath fallback = AllocatedPath::Build(home, ".config");
+		auto fallback = home / Path::FromFS(".config");
 		if (IsValidDir(fallback.c_str()))
 			return fallback;
 	}
 
-	return AllocatedPath::Null();
+	return nullptr;
 #else
-	return AllocatedPath::Null();
+	return nullptr;
 #endif
 }
 
 AllocatedPath
 GetUserMusicDir() noexcept
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	return GetStandardDir(CSIDL_MYMUSIC);	
 #elif defined(USE_XDG)
 	return GetUserDir("XDG_MUSIC_DIR");
 #elif defined(ANDROID)
 	return Environment::getExternalStoragePublicDirectory("Music");
 #else
-	return AllocatedPath::Null();
+	return nullptr;
 #endif
 }
 
@@ -274,20 +268,20 @@ GetUserCacheDir() noexcept
 	// Check for $HOME/.cache
 	auto home = GetHomeDir();
 	if (!home.IsNull()) {
-		AllocatedPath fallback = AllocatedPath::Build(home, ".cache");
+		auto fallback = home / Path::FromFS(".cache");
 		if (IsValidDir(fallback.c_str()))
 			return fallback;
 	}
 
-	return AllocatedPath::Null();
+	return nullptr;
 #elif defined(ANDROID)
 	return context->GetCacheDir(Java::GetEnv());
 #else
-	return AllocatedPath::Null();
+	return nullptr;
 #endif
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 
 AllocatedPath
 GetSystemConfigDir() noexcept
@@ -303,11 +297,11 @@ GetAppBaseDir() noexcept
 
 	// Check for error
 	if (ret == 0)
-		return AllocatedPath::Null();
+		return nullptr;
 
 	// Check for truncation
 	if (ret == app.size() && GetLastError() == ERROR_INSUFFICIENT_BUFFER)
-		return AllocatedPath::Null();
+		return nullptr;
 
 	auto app_path = AllocatedPath::FromFS(app.data());
 	return app_path.GetDirectoryName().GetDirectoryName();
@@ -326,7 +320,7 @@ GetHomeDir() noexcept
 	if (pw.ReadByUid(getuid()))
 		return SafePathFromFS(pw->pw_dir);
 #endif
-	return AllocatedPath::Null();
+	return nullptr;
 }
 
 AllocatedPath
@@ -340,7 +334,7 @@ GetHomeDir(const char *user_name) noexcept
 	if (pw.ReadByName(user_name))
 		return SafePathFromFS(pw->pw_dir);
 #endif
-	return AllocatedPath::Null();
+	return nullptr;
 }
 
 #endif
