@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,48 +19,51 @@
 
 #include "config.h"
 #include "AsyncInputStream.hxx"
+#include "CondHandler.hxx"
 #include "tag/Tag.hxx"
 #include "thread/Cond.hxx"
-#include "IOThread.hxx"
+#include "event/Loop.hxx"
 
 #include <stdexcept>
 
 #include <assert.h>
 #include <string.h>
 
-AsyncInputStream::AsyncInputStream(const char *_url,
-				   Mutex &_mutex, Cond &_cond,
+AsyncInputStream::AsyncInputStream(EventLoop &event_loop, const char *_url,
+				   Mutex &_mutex,
 				   size_t _buffer_size,
 				   size_t _resume_at)
-	:InputStream(_url, _mutex, _cond),
-	 deferred_resume(io_thread_get(), BIND_THIS_METHOD(DeferredResume)),
-	 deferred_seek(io_thread_get(), BIND_THIS_METHOD(DeferredSeek)),
+	:InputStream(_url, _mutex),
+	 deferred_resume(event_loop, BIND_THIS_METHOD(DeferredResume)),
+	 deferred_seek(event_loop, BIND_THIS_METHOD(DeferredSeek)),
 	 allocation(_buffer_size),
-	 buffer((uint8_t *)allocation.get(), _buffer_size),
-	 resume_at(_resume_at),
-	 open(true),
-	 paused(false),
-	 seek_state(SeekState::NONE),
-	 tag(nullptr) {}
+	 buffer(&allocation.front(), allocation.size()),
+	 resume_at(_resume_at)
+{
+	allocation.ForkCow(false);
+}
 
 AsyncInputStream::~AsyncInputStream()
 {
-	delete tag;
-
 	buffer.Clear();
 }
 
 void
-AsyncInputStream::SetTag(Tag *_tag) noexcept
+AsyncInputStream::SetTag(std::unique_ptr<Tag> _tag) noexcept
 {
-	delete tag;
-	tag = _tag;
+	tag = std::move(_tag);
+}
+
+void
+AsyncInputStream::ClearTag() noexcept
+{
+	tag.reset();
 }
 
 void
 AsyncInputStream::Pause() noexcept
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 
 	paused = true;
 }
@@ -68,7 +71,7 @@ AsyncInputStream::Pause() noexcept
 inline void
 AsyncInputStream::Resume()
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 
 	if (paused) {
 		paused = false;
@@ -80,18 +83,16 @@ AsyncInputStream::Resume()
 void
 AsyncInputStream::Check()
 {
-	if (postponed_exception) {
-		auto e = std::move(postponed_exception);
-		postponed_exception = std::exception_ptr();
-		std::rethrow_exception(e);
-	}
+	if (postponed_exception)
+		std::rethrow_exception(std::exchange(postponed_exception,
+						     std::exception_ptr()));
 }
 
 bool
 AsyncInputStream::IsEOF() noexcept
 {
 	return (KnownSize() && offset >= size) ||
-		(!open && buffer.IsEmpty());
+		(!open && buffer.empty());
 }
 
 void
@@ -111,7 +112,7 @@ AsyncInputStream::Seek(offset_type new_offset)
 
 	while (new_offset > offset) {
 		auto r = buffer.Read();
-		if (r.IsEmpty())
+		if (r.empty())
 			break;
 
 		const size_t nbytes =
@@ -133,8 +134,10 @@ AsyncInputStream::Seek(offset_type new_offset)
 
 	deferred_seek.Schedule();
 
+	CondInputStreamHandler cond_handler;
+	const ScopeExchangeInputStreamHandler h(*this, &cond_handler);
 	while (seek_state != SeekState::NONE)
-		cond.wait(mutex);
+		cond_handler.cond.wait(mutex);
 
 	Check();
 }
@@ -142,7 +145,7 @@ AsyncInputStream::Seek(offset_type new_offset)
 void
 AsyncInputStream::SeekDone() noexcept
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 	assert(IsSeekPending());
 
 	/* we may have reached end-of-file previously, and the
@@ -151,15 +154,13 @@ AsyncInputStream::SeekDone() noexcept
 	open = true;
 
 	seek_state = SeekState::NONE;
-	cond.broadcast();
+	InvokeOnAvailable();
 }
 
-Tag *
+std::unique_ptr<Tag>
 AsyncInputStream::ReadTag()
 {
-	Tag *result = tag;
-	tag = nullptr;
-	return result;
+	return std::exchange(tag, nullptr);
 }
 
 bool
@@ -167,13 +168,15 @@ AsyncInputStream::IsAvailable() noexcept
 {
 	return postponed_exception ||
 		IsEOF() ||
-		!buffer.IsEmpty();
+		!buffer.empty();
 }
 
 size_t
 AsyncInputStream::Read(void *ptr, size_t read_size)
 {
-	assert(!io_thread_inside());
+	assert(!GetEventLoop().IsInside());
+
+	CondInputStreamHandler cond_handler;
 
 	/* wait for data */
 	CircularBuffer<uint8_t>::Range r;
@@ -181,10 +184,11 @@ AsyncInputStream::Read(void *ptr, size_t read_size)
 		Check();
 
 		r = buffer.Read();
-		if (!r.IsEmpty() || IsEOF())
+		if (!r.empty() || IsEOF())
 			break;
 
-		cond.wait(mutex);
+		const ScopeExchangeInputStreamHandler h(*this, &cond_handler);
+		cond_handler.cond.wait(mutex);
 	}
 
 	const size_t nbytes = std::min(read_size, r.size);
@@ -207,14 +211,14 @@ AsyncInputStream::CommitWriteBuffer(size_t nbytes) noexcept
 	if (!IsReady())
 		SetReady();
 	else
-		cond.broadcast();
+		InvokeOnAvailable();
 }
 
 void
 AsyncInputStream::AppendToBuffer(const void *data, size_t append_size) noexcept
 {
 	auto w = buffer.Write();
-	assert(!w.IsEmpty());
+	assert(!w.empty());
 
 	size_t nbytes = std::min(w.size, append_size);
 	memcpy(w.data, data, nbytes);
@@ -223,7 +227,7 @@ AsyncInputStream::AppendToBuffer(const void *data, size_t append_size) noexcept
 	const size_t remaining = append_size - nbytes;
 	if (remaining > 0) {
 		w = buffer.Write();
-		assert(!w.IsEmpty());
+		assert(!w.empty());
 		assert(w.size >= remaining);
 
 		memcpy(w.data, (const uint8_t *)data + nbytes, remaining);
@@ -233,7 +237,7 @@ AsyncInputStream::AppendToBuffer(const void *data, size_t append_size) noexcept
 	if (!IsReady())
 		SetReady();
 	else
-		cond.broadcast();
+		InvokeOnAvailable();
 }
 
 void
@@ -245,7 +249,7 @@ AsyncInputStream::DeferredResume() noexcept
 		Resume();
 	} catch (...) {
 		postponed_exception = std::current_exception();
-		cond.broadcast();
+		InvokeOnAvailable();
 	}
 }
 
@@ -267,6 +271,6 @@ AsyncInputStream::DeferredSeek() noexcept
 	} catch (...) {
 		seek_state = SeekState::NONE;
 		postponed_exception = std::current_exception();
-		cond.broadcast();
+		InvokeOnAvailable();
 	}
 }
