@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,14 +19,15 @@
 
 #include "config.h"
 #include "InputStream.hxx"
-#include "thread/Cond.hxx"
+#include "Handler.hxx"
+#include "tag/Tag.hxx"
 #include "util/ASCII.hxx"
 
 #include <stdexcept>
 
 #include <assert.h>
 
-InputStream::~InputStream()
+InputStream::~InputStream() noexcept
 {
 }
 
@@ -36,36 +37,18 @@ InputStream::Check()
 }
 
 void
-InputStream::Update()
+InputStream::Update() noexcept
 {
 }
 
 void
-InputStream::SetReady()
+InputStream::SetReady() noexcept
 {
 	assert(!ready);
 
 	ready = true;
-	cond.broadcast();
-}
 
-void
-InputStream::WaitReady()
-{
-	while (true) {
-		Update();
-		if (ready)
-			break;
-
-		cond.wait(mutex);
-	}
-}
-
-void
-InputStream::LockWaitReady()
-{
-	const std::lock_guard<Mutex> protect(mutex);
-	WaitReady();
+	InvokeOnReady();
 }
 
 /**
@@ -78,6 +61,8 @@ static bool
 ExpensiveSeeking(const char *uri) noexcept
 {
 	return StringStartsWithCaseASCII(uri, "http://") ||
+		StringStartsWithCaseASCII(uri, "tidal://") ||
+		StringStartsWithCaseASCII(uri, "qobuz://") ||
 		StringStartsWithCaseASCII(uri, "https://");
 }
 
@@ -107,13 +92,13 @@ InputStream::LockSkip(offset_type _offset)
 	Skip(_offset);
 }
 
-Tag *
+std::unique_ptr<Tag>
 InputStream::ReadTag()
 {
 	return nullptr;
 }
 
-Tag *
+std::unique_ptr<Tag>
 InputStream::LockReadTag()
 {
 	const std::lock_guard<Mutex> protect(mutex);
@@ -174,3 +159,17 @@ InputStream::LockIsEOF() noexcept
 	const std::lock_guard<Mutex> protect(mutex);
 	return IsEOF();
 }
+
+void
+InputStream::InvokeOnReady() noexcept
+{
+	if (handler != nullptr)
+		handler->OnInputStreamReady();
+}
+
+void
+InputStream::InvokeOnAvailable() noexcept
+{
+	if (handler != nullptr)
+		handler->OnInputStreamAvailable();
+}
