@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,17 +19,15 @@
 
 #include "config.h"
 #include "Registry.hxx"
+#include "InputPlugin.hxx"
 #include "util/Macros.hxx"
-#include "plugins/FileInputPlugin.hxx"
+#include "plugins/TidalInputPlugin.hxx"
+#include "plugins/QobuzInputPlugin.hxx"
 
 #ifdef ENABLE_ALSA
 #include "plugins/AlsaInputPlugin.hxx"
 #endif
 
-#ifdef ENABLE_ARCHIVE
-#include "plugins/ArchiveInputPlugin.hxx"
-#endif
-
 #ifdef ENABLE_CURL
 #include "plugins/CurlInputPlugin.hxx"
 #endif
@@ -55,12 +53,14 @@
 #endif
 
 const InputPlugin *const input_plugins[] = {
-	&input_plugin_file,
 #ifdef ENABLE_ALSA
 	&input_plugin_alsa,
 #endif
-#ifdef ENABLE_ARCHIVE
-	&input_plugin_archive,
+#ifdef ENABLE_TIDAL
+	&tidal_input_plugin,
+#endif
+#ifdef ENABLE_QOBUZ
+	&qobuz_input_plugin,
 #endif
 #ifdef ENABLE_CURL
 	&input_plugin_curl,
@@ -84,3 +84,14 @@ const InputPlugin *const input_plugins[] = {
 };
 
 bool input_plugins_enabled[ARRAY_SIZE(input_plugins) - 1];
+
+bool
+HasRemoteTagScanner(const char *uri) noexcept
+{
+	input_plugins_for_each_enabled(plugin)
+		if (plugin->scan_tags != nullptr &&
+		    plugin->SupportsUri(uri))
+			return true;
+
+	return false;
+}
