@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,17 +22,20 @@
 #include "lib/curl/Error.hxx"
 #include "lib/curl/Easy.hxx"
 #include "lib/curl/Global.hxx"
+#include "lib/curl/Init.hxx"
 #include "lib/curl/Request.hxx"
 #include "lib/curl/Handler.hxx"
 #include "lib/curl/Slist.hxx"
+#include "../MaybeBufferedInputStream.hxx"
 #include "../AsyncInputStream.hxx"
 #include "../IcyInputStream.hxx"
+#include "IcyMetaDataParser.hxx"
 #include "../InputPlugin.hxx"
-#include "config/ConfigGlobal.hxx"
 #include "config/Block.hxx"
-#include "tag/TagBuilder.hxx"
+#include "tag/Builder.hxx"
+#include "tag/Tag.hxx"
 #include "event/Call.hxx"
-#include "IOThread.hxx"
+#include "event/Loop.hxx"
 #include "util/ASCII.hxx"
 #include "util/StringUtil.hxx"
 #include "util/StringFormat.hxx"
@@ -65,7 +68,7 @@ static const size_t CURL_MAX_BUFFERED = 512 * 1024;
  */
 static const size_t CURL_RESUME_AT = 384 * 1024;
 
-struct CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
+class CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
 	/* some buffers which were passed to libcurl, which we have
 	   too free */
 	CurlSlist request_headers;
@@ -73,22 +76,25 @@ struct CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
 	CurlRequest *request = nullptr;
 
 	/** parser for icy-metadata */
-	IcyInputStream *icy;
+	std::shared_ptr<IcyMetaDataParser> icy;
 
-	CurlInputStream(const char *_url, Mutex &_mutex, Cond &_cond)
-		:AsyncInputStream(_url, _mutex, _cond,
-				  CURL_MAX_BUFFERED,
-				  CURL_RESUME_AT),
-		 icy(new IcyInputStream(this)) {
-	}
+public:
+	template<typename I>
+	CurlInputStream(EventLoop &event_loop, const char *_url,
+			const std::multimap<std::string, std::string> &headers,
+			I &&_icy,
+			Mutex &_mutex);
 
-	~CurlInputStream();
+	~CurlInputStream() noexcept;
 
 	CurlInputStream(const CurlInputStream &) = delete;
 	CurlInputStream &operator=(const CurlInputStream &) = delete;
 
-	static InputStream *Open(const char *url, Mutex &mutex, Cond &cond);
+	static InputStreamPtr Open(const char *url,
+				   const std::multimap<std::string, std::string> &headers,
+				   Mutex &mutex);
 
+private:
 	/**
 	 * Create and initialize a new #CurlRequest instance.  After
 	 * this, you may add more request headers and set options.  To
@@ -108,7 +114,7 @@ struct CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
 	 *
 	 * Runs in the I/O thread.
 	 */
-	void FreeEasy();
+	void FreeEasy() noexcept;
 
 	/**
 	 * Frees the current "libcurl easy" handle, and everything associated
@@ -116,7 +122,7 @@ struct CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
 	 *
 	 * The mutex must not be locked.
 	 */
-	void FreeEasyIndirect();
+	void FreeEasyIndirect() noexcept;
 
 	/**
 	 * The DoSeek() implementation invoked in the IOThread.
@@ -128,7 +134,7 @@ struct CurlInputStream final : public AsyncInputStream, CurlResponseHandler {
 		       std::multimap<std::string, std::string> &&headers) override;
 	void OnData(ConstBuffer<void> data) override;
 	void OnEnd() override;
-	void OnError(std::exception_ptr e) override;
+	void OnError(std::exception_ptr e) noexcept override;
 
 	/* virtual methods from AsyncInputStream */
 	virtual void DoResume() override;
@@ -144,14 +150,14 @@ static unsigned proxy_port;
 
 static bool verify_peer, verify_host;
 
-static CurlGlobal *curl_global;
+static CurlInit *curl_init;
 
 static constexpr Domain curl_domain("curl");
 
 void
 CurlInputStream::DoResume()
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 
 	mutex.unlock();
 	request->Resume();
@@ -159,25 +165,23 @@ CurlInputStream::DoResume()
 }
 
 void
-CurlInputStream::FreeEasy()
+CurlInputStream::FreeEasy() noexcept
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 
 	if (request == nullptr)
 		return;
 
 	delete request;
 	request = nullptr;
-
-	request_headers.Clear();
 }
 
 void
-CurlInputStream::FreeEasyIndirect()
+CurlInputStream::FreeEasyIndirect() noexcept
 {
-	BlockingCall(io_thread_get(), [this](){
+	BlockingCall(GetEventLoop(), [this](){
 			FreeEasy();
-			curl_global->InvalidateSockets();
+			(*curl_init)->InvalidateSockets();
 		});
 }
 
@@ -185,8 +189,9 @@ void
 CurlInputStream::OnHeaders(unsigned status,
 			   std::multimap<std::string, std::string> &&headers)
 {
-	assert(io_thread_inside());
+	assert(GetEventLoop().IsInside());
 	assert(!postponed_exception);
+	assert(!icy || !icy->IsDefined());
 
 	if (status < 200 || status >= 300)
 		throw HttpStatusError(status,
@@ -201,9 +206,7 @@ CurlInputStream::OnHeaders(unsigned status,
 		return;
 	}
 
-	if (!icy->IsEnabled() &&
-	    headers.find("accept-ranges") != headers.end())
-		/* a stream with icy-metadata is not seekable */
+	if (headers.find("accept-ranges") != headers.end())
 		seekable = true;
 
 	auto i = headers.find("content-length");
@@ -228,7 +231,7 @@ CurlInputStream::OnHeaders(unsigned status,
 		SetTag(tag_builder.CommitNew());
 	}
 
-	if (!icy->IsEnabled()) {
+	if (icy) {
 		i = headers.find("icy-metaint");
 
 		if (i != headers.end()) {
@@ -239,7 +242,7 @@ CurlInputStream::OnHeaders(unsigned status,
 #endif
 
 			if (icy_metaint > 0) {
-				icy->Enable(icy_metaint);
+				icy->Start(icy_metaint);
 
 				/* a stream with icy-metadata is not
 				   seekable */
@@ -273,13 +276,13 @@ void
 CurlInputStream::OnEnd()
 {
 	const std::lock_guard<Mutex> protect(mutex);
-	cond.broadcast();
+	InvokeOnAvailable();
 
 	AsyncInputStream::SetClosed();
 }
 
 void
-CurlInputStream::OnError(std::exception_ptr e)
+CurlInputStream::OnError(std::exception_ptr e) noexcept
 {
 	const std::lock_guard<Mutex> protect(mutex);
 	postponed_exception = std::move(e);
@@ -289,7 +292,7 @@ CurlInputStream::OnError(std::exception_ptr e)
 	else if (!IsReady())
 		SetReady();
 	else
-		cond.broadcast();
+		InvokeOnAvailable();
 
 	AsyncInputStream::SetClosed();
 }
@@ -300,11 +303,14 @@ CurlInputStream::OnError(std::exception_ptr e)
  */
 
 static void
-input_curl_init(const ConfigBlock &block)
+input_curl_init(EventLoop &event_loop, const ConfigBlock &block)
 {
-	CURLcode code = curl_global_init(CURL_GLOBAL_ALL);
-	if (code != CURLE_OK)
-		throw PluginUnavailable(curl_easy_strerror(code));
+	try {
+		curl_init = new CurlInit(event_loop);
+	} catch (const std::runtime_error &e) {
+		LogError(e);
+		throw PluginUnavailable(e.what());
+	}
 
 	const auto version_info = curl_version_info(CURLVERSION_FIRST);
 	if (version_info != nullptr) {
@@ -321,42 +327,37 @@ input_curl_init(const ConfigBlock &block)
 	proxy_user = block.GetBlockValue("proxy_user");
 	proxy_password = block.GetBlockValue("proxy_password");
 
-	if (proxy == nullptr) {
-		/* deprecated proxy configuration */
-		proxy = config_get_string(ConfigOption::HTTP_PROXY_HOST);
-		proxy_port = config_get_positive(ConfigOption::HTTP_PROXY_PORT, 0);
-		proxy_user = config_get_string(ConfigOption::HTTP_PROXY_USER);
-		proxy_password = config_get_string(ConfigOption::HTTP_PROXY_PASSWORD,
-						   "");
-	}
-
 	verify_peer = block.GetBlockValue("verify_peer", true);
 	verify_host = block.GetBlockValue("verify_host", true);
-
-	try {
-		curl_global = new CurlGlobal(io_thread_get());
-	} catch (const std::runtime_error &e) {
-		LogError(e);
-		curl_slist_free_all(http_200_aliases);
-		curl_global_cleanup();
-		throw PluginUnavailable("curl_multi_init() failed");
-	}
 }
 
 static void
-input_curl_finish(void)
+input_curl_finish() noexcept
 {
-	BlockingCall(io_thread_get(), [](){
-			delete curl_global;
-		});
+	delete curl_init;
 
 	curl_slist_free_all(http_200_aliases);
 	http_200_aliases = nullptr;
-
-	curl_global_cleanup();
 }
 
-CurlInputStream::~CurlInputStream()
+template<typename I>
+inline
+CurlInputStream::CurlInputStream(EventLoop &event_loop, const char *_url,
+				 const std::multimap<std::string, std::string> &headers,
+				 I &&_icy,
+				 Mutex &_mutex)
+	:AsyncInputStream(event_loop, _url, _mutex,
+			  CURL_MAX_BUFFERED,
+			  CURL_RESUME_AT),
+	 icy(std::forward<I>(_icy))
+{
+	request_headers.Append("Icy-Metadata: 1");
+
+	for (const auto &i : headers)
+		request_headers.Append((i.first + ":" + i.second).c_str());
+}
+
+CurlInputStream::~CurlInputStream() noexcept
 {
 	FreeEasyIndirect();
 }
@@ -364,7 +365,7 @@ CurlInputStream::~CurlInputStream()
 void
 CurlInputStream::InitEasy()
 {
-	request = new CurlRequest(*curl_global, GetURI(), *this);
+	request = new CurlRequest(**curl_init, GetURI(), *this);
 
 	request->SetOption(CURLOPT_HTTP200ALIASES, http_200_aliases);
 	request->SetOption(CURLOPT_FOLLOWLOCATION, 1l);
@@ -384,16 +385,12 @@ CurlInputStream::InitEasy()
 
 	request->SetOption(CURLOPT_SSL_VERIFYPEER, verify_peer ? 1l : 0l);
 	request->SetOption(CURLOPT_SSL_VERIFYHOST, verify_host ? 2l : 0l);
-
-	request_headers.Clear();
-	request_headers.Append("Icy-Metadata: 1");
+	request->SetOption(CURLOPT_HTTPHEADER, request_headers.Get());
 }
 
 void
 CurlInputStream::StartRequest()
 {
-	request->SetOption(CURLOPT_HTTPHEADER, request_headers.Get());
-
 	request->Start();
 }
 
@@ -429,44 +426,62 @@ void
 CurlInputStream::DoSeek(offset_type new_offset)
 {
 	assert(IsReady());
+	assert(seekable);
 
 	const ScopeUnlock unlock(mutex);
 
-	BlockingCall(io_thread_get(), [this, new_offset](){
+	BlockingCall(GetEventLoop(), [this, new_offset](){
 			SeekInternal(new_offset);
 		});
 }
 
-inline InputStream *
-CurlInputStream::Open(const char *url, Mutex &mutex, Cond &cond)
+inline InputStreamPtr
+CurlInputStream::Open(const char *url,
+		      const std::multimap<std::string, std::string> &headers,
+		      Mutex &mutex)
 {
-	CurlInputStream *c = new CurlInputStream(url, mutex, cond);
+	auto icy = std::make_shared<IcyMetaDataParser>();
 
-	try {
-		BlockingCall(io_thread_get(), [c](){
-				c->InitEasy();
-				c->StartRequest();
-			});
-	} catch (...) {
-		delete c;
-		throw;
-	}
+	auto c = std::make_unique<CurlInputStream>((*curl_init)->GetEventLoop(),
+						   url, headers,
+						   icy,
+						   mutex);
 
-	return c->icy;
+	BlockingCall(c->GetEventLoop(), [&c](){
+			c->InitEasy();
+			c->StartRequest();
+		});
+
+	return std::make_unique<MaybeBufferedInputStream>(std::make_unique<IcyInputStream>(std::move(c), std::move(icy)));
 }
 
-static InputStream *
-input_curl_open(const char *url, Mutex &mutex, Cond &cond)
+InputStreamPtr
+OpenCurlInputStream(const char *uri,
+		    const std::multimap<std::string, std::string> &headers,
+		    Mutex &mutex)
+{
+	return CurlInputStream::Open(uri, headers, mutex);
+}
+
+static InputStreamPtr
+input_curl_open(const char *url, Mutex &mutex)
 {
 	if (!StringStartsWithCaseASCII(url, "http://") &&
 	    !StringStartsWithCaseASCII(url, "https://"))
 		return nullptr;
 
-	return CurlInputStream::Open(url, mutex, cond);
+	return CurlInputStream::Open(url, {}, mutex);
 }
 
+static constexpr const char *curl_prefixes[] = {
+	"http://",
+	"https://",
+	nullptr
+};
+
 const struct InputPlugin input_plugin_curl = {
 	"curl",
+	curl_prefixes,
 	input_curl_init,
 	input_curl_finish,
 	input_curl_open,
