@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,9 +37,9 @@ class SmbclientInputStream final : public InputStream {
 
 public:
 	SmbclientInputStream(const char *_uri,
-			     Mutex &_mutex, Cond &_cond,
+			     Mutex &_mutex,
 			     SMBCCTX *_ctx, int _fd, const struct stat &st)
-		:InputStream(_uri, _mutex, _cond),
+		:InputStream(_uri, _mutex),
 		 ctx(_ctx), fd(_fd) {
 		seekable = true;
 		size = st.st_size;
@@ -69,7 +69,7 @@ public:
  */
 
 static void
-input_smbclient_init(gcc_unused const ConfigBlock &block)
+input_smbclient_init(EventLoop &, const ConfigBlock &)
 {
 	try {
 		SmbclientInit();
@@ -83,13 +83,10 @@ input_smbclient_init(gcc_unused const ConfigBlock &block)
 	// TODO: evaluate ConfigBlock, call smbc_setOption*()
 }
 
-static InputStream *
+static InputStreamPtr
 input_smbclient_open(const char *uri,
-		     Mutex &mutex, Cond &cond)
+		     Mutex &mutex)
 {
-	if (!StringStartsWithCaseASCII(uri, "smb://"))
-		return nullptr;
-
 	const std::lock_guard<Mutex> protect(smbclient_mutex);
 
 	SMBCCTX *ctx = smbc_new_context();
@@ -119,7 +116,8 @@ input_smbclient_open(const char *uri,
 		throw MakeErrno(e, "smbc_fstat() failed");
 	}
 
-	return new SmbclientInputStream(uri, mutex, cond, ctx, fd, st);
+	return std::make_unique<SmbclientInputStream>(uri, mutex,
+						      ctx, fd, st);
 }
 
 size_t
@@ -157,8 +155,14 @@ SmbclientInputStream::Seek(offset_type new_offset)
 	offset = result;
 }
 
+static constexpr const char *smbclient_prefixes[] = {
+	"smb://",
+	nullptr
+};
+
 const InputPlugin input_plugin_smbclient = {
 	"smbclient",
+	smbclient_prefixes,
 	input_smbclient_init,
 	nullptr,
 	input_smbclient_open,
