@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,8 +32,8 @@ ExpatParser::Parse(InputStream &is)
 		if (nbytes == 0)
 			break;
 
-		Parse(buffer, nbytes, false);
+		Parse(buffer, nbytes);
 	}
 
-	Parse("", 0, true);
+	CompleteParse();
 }
