@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ public:
 	void Open() override {
 	}
 
-	void Close() override {
+	void Close() noexcept override {
 	}
 
 	int GetVolume() override;
