@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
 #include "config.h"
 #include "ConfiguredResampler.hxx"
 #include "FallbackResampler.hxx"
-#include "config/ConfigGlobal.hxx"
-#include "config/ConfigOption.hxx"
-#include "config/ConfigError.hxx"
+#include "config/Data.hxx"
+#include "config/Option.hxx"
+#include "config/Domain.hxx"
 #include "config/Block.hxx"
 #include "config/Param.hxx"
 #include "util/RuntimeError.hxx"
@@ -53,7 +53,7 @@ enum class SelectedResampler {
 static SelectedResampler selected_resampler = SelectedResampler::FALLBACK;
 
 static const ConfigBlock *
-MakeResamplerDefaultConfig(ConfigBlock &block)
+MakeResamplerDefaultConfig(ConfigBlock &block) noexcept
 {
 	assert(block.IsEmpty());
 
@@ -72,7 +72,7 @@ MakeResamplerDefaultConfig(ConfigBlock &block)
  * "resampler" block.
  */
 static const ConfigBlock *
-MigrateResamplerConfig(const ConfigParam &param, ConfigBlock &block)
+MigrateResamplerConfig(const ConfigParam &param, ConfigBlock &block) noexcept
 {
 	assert(block.IsEmpty());
 
@@ -103,7 +103,7 @@ MigrateResamplerConfig(const ConfigParam &param, ConfigBlock &block)
 }
 
 static const ConfigBlock *
-MigrateResamplerConfig(const ConfigParam *param, ConfigBlock &buffer)
+MigrateResamplerConfig(const ConfigParam *param, ConfigBlock &buffer) noexcept
 {
 	assert(buffer.IsEmpty());
 
@@ -113,11 +113,11 @@ MigrateResamplerConfig(const ConfigParam *param, ConfigBlock &buffer)
 }
 
 static const ConfigBlock *
-GetResamplerConfig(ConfigBlock &buffer)
+GetResamplerConfig(const ConfigData &config, ConfigBlock &buffer)
 {
 	const auto *old_param =
-		config_get_param(ConfigOption::SAMPLERATE_CONVERTER);
-	const auto *block = config_get_block(ConfigBlockOption::RESAMPLER);
+		config.GetParam(ConfigOption::SAMPLERATE_CONVERTER);
+	const auto *block = config.GetBlock(ConfigBlockOption::RESAMPLER);
 	if (block == nullptr)
 		return MigrateResamplerConfig(old_param, buffer);
 
@@ -125,14 +125,15 @@ GetResamplerConfig(ConfigBlock &buffer)
 		throw FormatRuntimeError("Cannot use both 'resampler' (line %d) and 'samplerate_converter' (line %d)",
 					 block->line, old_param->line);
 
+	block->SetUsed();
 	return block;
 }
 
 void
-pcm_resampler_global_init()
+pcm_resampler_global_init(const ConfigData &config)
 {
 	ConfigBlock buffer;
-	const auto *block = GetResamplerConfig(buffer);
+	const auto *block = GetResamplerConfig(config, buffer);
 
 	const char *plugin_name = block->GetBlockValue("plugin");
 	if (plugin_name == nullptr)
