@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,15 +20,13 @@
 #include "config.h"
 #include "FormatConverter.hxx"
 #include "PcmFormat.hxx"
-#include "Domain.hxx"
 #include "util/ConstBuffer.hxx"
-#include "util/Error.hxx"
+#include "util/RuntimeError.hxx"
 
 #include <assert.h>
 
-bool
-PcmFormatConverter::Open(SampleFormat _src_format, SampleFormat _dest_format,
-			 Error &error)
+void
+PcmFormatConverter::Open(SampleFormat _src_format, SampleFormat _dest_format)
 {
 	assert(_src_format != SampleFormat::UNDEFINED);
 	assert(_dest_format != SampleFormat::UNDEFINED);
@@ -40,11 +38,9 @@ PcmFormatConverter::Open(SampleFormat _src_format, SampleFormat _dest_format,
 
 	case SampleFormat::S8:
 	case SampleFormat::DSD:
-		error.Format(pcm_domain,
-			     "PCM conversion from %s to %s is not implemented",
-			     sample_format_to_string(_src_format),
-			     sample_format_to_string(_dest_format));
-		return false;
+		throw FormatRuntimeError("PCM conversion from %s to %s is not implemented",
+					 sample_format_to_string(_src_format),
+					 sample_format_to_string(_dest_format));
 
 	case SampleFormat::S16:
 	case SampleFormat::S24_P32:
@@ -55,11 +51,10 @@ PcmFormatConverter::Open(SampleFormat _src_format, SampleFormat _dest_format,
 
 	src_format = _src_format;
 	dest_format = _dest_format;
-	return true;
 }
 
 void
-PcmFormatConverter::Close()
+PcmFormatConverter::Close() noexcept
 {
 #ifndef NDEBUG
 	src_format = SampleFormat::UNDEFINED;
@@ -68,7 +63,7 @@ PcmFormatConverter::Close()
 }
 
 ConstBuffer<void>
-PcmFormatConverter::Convert(ConstBuffer<void> src, gcc_unused Error &error)
+PcmFormatConverter::Convert(ConstBuffer<void> src) noexcept
 {
 	switch (dest_format) {
 	case SampleFormat::UNDEFINED:
