@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,47 +20,42 @@
 #include "config.h"
 #include "PcmDsd.hxx"
 #include "dsd2pcm/dsd2pcm.h"
-#include "util/Macros.hxx"
 #include "util/ConstBuffer.hxx"
 
-#include <algorithm>
-
 #include <assert.h>
 
-PcmDsd::PcmDsd()
+PcmDsd::PcmDsd() noexcept
 {
-	std::fill_n(dsd2pcm, ARRAY_SIZE(dsd2pcm), nullptr);
+	dsd2pcm.fill(nullptr);
 }
 
-PcmDsd::~PcmDsd()
+PcmDsd::~PcmDsd() noexcept
 {
-	for (unsigned i = 0; i < ARRAY_SIZE(dsd2pcm); ++i)
-		if (dsd2pcm[i] != nullptr)
-			dsd2pcm_destroy(dsd2pcm[i]);
+	for (auto i : dsd2pcm)
+		if (i != nullptr)
+			dsd2pcm_destroy(i);
 }
 
 void
-PcmDsd::Reset()
+PcmDsd::Reset() noexcept
 {
-	for (unsigned i = 0; i < ARRAY_SIZE(dsd2pcm); ++i)
-		if (dsd2pcm[i] != nullptr)
-			dsd2pcm_reset(dsd2pcm[i]);
+	for (auto i : dsd2pcm)
+		if (i != nullptr)
+			dsd2pcm_reset(i);
 }
 
 ConstBuffer<float>
-PcmDsd::ToFloat(unsigned channels, ConstBuffer<uint8_t> src)
+PcmDsd::ToFloat(unsigned channels, ConstBuffer<uint8_t> src) noexcept
 {
 	assert(!src.IsNull());
-	assert(!src.IsEmpty());
+	assert(!src.empty());
 	assert(src.size % channels == 0);
-	assert(channels <= ARRAY_SIZE(dsd2pcm));
+	assert(channels <= dsd2pcm.max_size());
 
 	const unsigned num_samples = src.size;
 	const unsigned num_frames = src.size / channels;
 
-	float *dest;
-	const size_t dest_size = num_samples * sizeof(*dest);
-	dest = (float *)buffer.Get(dest_size);
+	float *dest = buffer.GetT<float>(num_samples);
 
 	for (unsigned c = 0; c < channels; ++c) {
 		if (dsd2pcm[c] == nullptr) {
