@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 #include <string.h>
 
 void
-PcmSilence(WritableBuffer<void> dest, SampleFormat format)
+PcmSilence(WritableBuffer<void> dest, SampleFormat format) noexcept
 {
 	uint8_t pattern = 0;
 	if (format == SampleFormat::DSD)
