@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,20 +21,21 @@
 #include "PlaylistAny.hxx"
 #include "PlaylistStream.hxx"
 #include "PlaylistMapper.hxx"
+#include "SongEnumerator.hxx"
 #include "util/UriUtil.hxx"
 
-SongEnumerator *
+std::unique_ptr<SongEnumerator>
 playlist_open_any(const char *uri,
 #ifdef ENABLE_DATABASE
 		  const Storage *storage,
 #endif
-		  Mutex &mutex, Cond &cond)
+		  Mutex &mutex)
 {
 	return uri_has_scheme(uri)
-		? playlist_open_remote(uri, mutex, cond)
+		? playlist_open_remote(uri, mutex)
 		: playlist_mapper_open(uri,
 #ifdef ENABLE_DATABASE
 				       storage,
 #endif
-				       mutex, cond);
+				       mutex);
 }
