@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 #include "AsxPlaylistPlugin.hxx"
 #include "../PlaylistPlugin.hxx"
 #include "../MemorySongEnumerator.hxx"
-#include "tag/TagBuilder.hxx"
+#include "tag/Builder.hxx"
 #include "util/ASCII.hxx"
 #include "util/StringView.hxx"
 #include "lib/expat/ExpatParser.hxx"
@@ -142,7 +142,7 @@ asx_char_data(void *user_data, const XML_Char *s, int len)
  *
  */
 
-static SongEnumerator *
+static std::unique_ptr<SongEnumerator>
 asx_open_stream(InputStreamPtr &&is)
 {
 	AsxParser parser;
@@ -155,7 +155,7 @@ asx_open_stream(InputStreamPtr &&is)
 	}
 
 	parser.songs.reverse();
-	return new MemorySongEnumerator(std::move(parser.songs));
+	return std::make_unique<MemorySongEnumerator>(std::move(parser.songs));
 }
 
 static const char *const asx_suffixes[] = {
