@@ -23,6 +23,7 @@
 #include "storage/StorageInterface.hxx"
 #include "storage/FileInfo.hxx"
 #include "storage/MemoryDirectoryReader.hxx"
+#include "lib/curl/Init.hxx"
 #include "lib/curl/Global.hxx"
 #include "lib/curl/Slist.hxx"
 #include "lib/curl/Request.hxx"
@@ -30,10 +31,11 @@
 #include "lib/expat/ExpatParser.hxx"
 #include "fs/Traits.hxx"
 #include "event/Call.hxx"
-#include "event/DeferredMonitor.hxx"
+#include "event/DeferEvent.hxx"
 #include "thread/Mutex.hxx"
 #include "thread/Cond.hxx"
 #include "util/ASCII.hxx"
+#include "util/ChronoUtil.hxx"
 #include "util/IterableSplitString.hxx"
 #include "util/RuntimeError.hxx"
 #include "util/StringCompare.hxx"
@@ -51,21 +53,17 @@
 class CurlStorage final : public Storage {
 	const std::string base;
 
-	CurlGlobal *const curl;
+	CurlInit curl;
 
 public:
 	CurlStorage(EventLoop &_loop, const char *_base)
 		:base(_base),
-		 curl(new CurlGlobal(_loop)) {}
-
-	~CurlStorage() {
-		BlockingCall(curl->GetEventLoop(), [this](){ delete curl; });
-	}
+		 curl(_loop) {}
 
 	/* virtual methods from class Storage */
 	StorageFileInfo GetInfo(const char *uri_utf8, bool follow) override;
 
-	StorageDirectoryReader *OpenDirectory(const char *uri_utf8) override;
+	std::unique_ptr<StorageDirectoryReader> OpenDirectory(const char *uri_utf8) override;
 
 	std::string MapUTF8(const char *uri_utf8) const noexcept override;
 
@@ -102,7 +100,9 @@ CurlStorage::MapToRelativeUTF8(const char *uri_utf8) const noexcept
 	return PathTraitsUTF8::Relative(base.c_str(), uri_utf8);
 }
 
-class BlockingHttpRequest : protected CurlResponseHandler, DeferredMonitor {
+class BlockingHttpRequest : protected CurlResponseHandler {
+	DeferEvent defer_start;
+
 	std::exception_ptr postponed_error;
 
 	bool done = false;
@@ -115,12 +115,13 @@ protected:
 
 public:
 	BlockingHttpRequest(CurlGlobal &curl, const char *uri)
-		:DeferredMonitor(curl.GetEventLoop()),
+		:defer_start(curl.GetEventLoop(),
+			     BIND_THIS_METHOD(OnDeferredStart)),
 		 request(curl, uri, *this) {
 		// TODO: use CurlInputStream's configuration
 
 		/* start the transfer inside the IOThread */
-		DeferredMonitor::Schedule();
+		defer_start.Schedule();
 	}
 
 	void Wait() {
@@ -147,56 +148,25 @@ protected:
 	}
 
 private:
-	/* virtual methods from DeferredMonitor */
-	void RunDeferred() final {
+	/* DeferEvent callback */
+	void OnDeferredStart() noexcept {
 		assert(!done);
 
-		request.Start();
+		try {
+			request.Start();
+		} catch (...) {
+			OnError(std::current_exception());
+		}
 	}
 
 	/* virtual methods from CurlResponseHandler */
-	void OnError(std::exception_ptr e) final {
+	void OnError(std::exception_ptr e) noexcept final {
 		const std::lock_guard<Mutex> lock(mutex);
 		postponed_error = std::move(e);
 		SetDone();
 	}
 };
 
-/**
- * A helper class which feeds a (foreign) memory buffer into the
- * CURLOPT_READFUNCTION.
- */
-class CurlRequestBody {
-	ConstBuffer<char> data;
-
-public:
-	explicit CurlRequestBody(ConstBuffer<void> _data)
-		:data(ConstBuffer<char>::FromVoid(_data)) {}
-
-	explicit constexpr CurlRequestBody(StringView _data)
-		:data(_data) {}
-
-	template<typename T>
-	CurlRequestBody(CurlRequest &request, T _data)
-		:CurlRequestBody(_data) {
-		request.SetOption(CURLOPT_READFUNCTION, Callback);
-		request.SetOption(CURLOPT_READDATA, this);
-	}
-
-private:
-	size_t Read(char *buffer, size_t size) {
-		size_t n = std::min(size, data.size);
-		std::copy_n(data.begin(), n, buffer);
-		return n;
-	}
-
-	static size_t Callback(char *buffer, size_t size, size_t nitems,
-			       void *instream) {
-		auto &rb = *(CurlRequestBody *)instream;
-		return rb.Read(buffer, size * nitems);
-	}
-};
-
 /**
  * The (relevant) contents of a "<D:response>" element.
  */
@@ -236,7 +206,7 @@ ParseTimeStamp(const char *s)
 	try {
 		// TODO: make this more robust
 		return ParseTimePoint(s, "%a, %d %b %Y %T %Z");
-	} catch (const std::runtime_error &) {
+	} catch (...) {
 		return std::chrono::system_clock::time_point::min();
 	}
 }
@@ -281,7 +251,6 @@ IsXmlContentType(const std::multimap<std::string, std::string> &headers) noexcep
  */
 class PropfindOperation : BlockingHttpRequest, CommonExpatParser {
 	CurlSlist request_headers;
-	CurlRequestBody request_body;
 
 	enum class State {
 		ROOT,
@@ -298,13 +267,7 @@ class PropfindOperation : BlockingHttpRequest, CommonExpatParser {
 public:
 	PropfindOperation(CurlGlobal &_curl, const char *_uri, unsigned depth)
 		:BlockingHttpRequest(_curl, _uri),
-		 CommonExpatParser(ExpatNamespaceSeparator{'|'}),
-		 request_body(request,
-			      "<?xml version=\"1.0\"?>\n"
-			      "<a:propfind xmlns:a=\"DAV:\">"
-			      "<a:prop><a:getcontenttype/></a:prop>"
-			      "<a:prop><a:getcontentlength/></a:prop>"
-			      "</a:propfind>")
+		 CommonExpatParser(ExpatNamespaceSeparator{'|'})
 	{
 		request.SetOption(CURLOPT_CUSTOMREQUEST, "PROPFIND");
 
@@ -312,6 +275,13 @@ public:
 
 		request.SetOption(CURLOPT_HTTPHEADER, request_headers.Get());
 
+		request.SetOption(CURLOPT_POSTFIELDS,
+				  "<?xml version=\"1.0\"?>\n"
+				  "<a:propfind xmlns:a=\"DAV:\">"
+				  "<a:prop><a:getcontenttype/></a:prop>"
+				  "<a:prop><a:getcontentlength/></a:prop>"
+				  "</a:propfind>");
+
 		// TODO: send request body
 	}
 
@@ -340,11 +310,11 @@ private:
 
 	void OnData(ConstBuffer<void> _data) final {
 		const auto data = ConstBuffer<char>::FromVoid(_data);
-		Parse(data.data, data.size, false);
+		Parse(data.data, data.size);
 	}
 
 	void OnEnd() final {
-		Parse("", 0, true);
+		CompleteParse();
 		LockSetDone();
 	}
 
@@ -457,11 +427,8 @@ class HttpGetInfoOperation final : public PropfindOperation {
 
 public:
 	HttpGetInfoOperation(CurlGlobal &curl, const char *uri)
-		:PropfindOperation(curl, uri, 0) {
-		info.type = StorageFileInfo::Type::OTHER;
-		info.size = 0;
-		info.mtime = 0;
-		info.device = info.inode = 0;
+		:PropfindOperation(curl, uri, 0),
+		 info(StorageFileInfo::Type::OTHER) {
 	}
 
 	const StorageFileInfo &Perform() {
@@ -479,10 +446,7 @@ protected:
 			? StorageFileInfo::Type::DIRECTORY
 			: StorageFileInfo::Type::REGULAR;
 		info.size = r.length;
-		info.mtime = r.mtime > std::chrono::system_clock::time_point()
-			? std::chrono::system_clock::to_time_t(r.mtime)
-			: 0;
-		info.device = info.inode = 0;
+		info.mtime = r.mtime;
 	}
 };
 
@@ -520,14 +484,14 @@ public:
 		:PropfindOperation(curl, uri, 1),
 		 base_path(UriPathOrSlash(uri)) {}
 
-	StorageDirectoryReader *Perform() {
+	std::unique_ptr<StorageDirectoryReader> Perform() {
 		Wait();
 		return ToReader();
 	}
 
 private:
-	StorageDirectoryReader *ToReader() {
-		return new MemoryStorageDirectoryReader(std::move(entries));
+	std::unique_ptr<StorageDirectoryReader> ToReader() {
+		return std::make_unique<MemoryStorageDirectoryReader>(std::move(entries));
 	}
 
 	/**
@@ -572,18 +536,15 @@ protected:
 		entries.emplace_front(std::string(name.data, name.size));
 
 		auto &info = entries.front().info;
-		info.type = r.collection
-			? StorageFileInfo::Type::DIRECTORY
-			: StorageFileInfo::Type::REGULAR;
+		info = StorageFileInfo(r.collection
+				       ? StorageFileInfo::Type::DIRECTORY
+				       : StorageFileInfo::Type::REGULAR);
 		info.size = r.length;
-		info.mtime = r.mtime > std::chrono::system_clock::time_point()
-			? std::chrono::system_clock::to_time_t(r.mtime)
-			: 0;
-		info.device = info.inode = 0;
+		info.mtime = r.mtime;
 	}
 };
 
-StorageDirectoryReader *
+std::unique_ptr<StorageDirectoryReader>
 CurlStorage::OpenDirectory(const char *uri_utf8)
 {
 	// TODO: escape the given URI
@@ -598,14 +559,14 @@ CurlStorage::OpenDirectory(const char *uri_utf8)
 	return HttpListDirectoryOperation(*curl, uri.c_str()).Perform();
 }
 
-static Storage *
+static std::unique_ptr<Storage>
 CreateCurlStorageURI(EventLoop &event_loop, const char *uri)
 {
 	if (!StringStartsWithCaseASCII(uri, "http://") &&
 	    !StringStartsWithCaseASCII(uri, "https://"))
 		return nullptr;
 
-	return new CurlStorage(event_loop, uri);
+	return std::make_unique<CurlStorage>(event_loop, uri);
 }
 
 const StoragePlugin curl_storage_plugin = {
