@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,8 +20,10 @@
 #include "config.h"
 #include "Format.hxx"
 #include "Tag.hxx"
+#include "ParseName.hxx"
 #include "util/format.h"
-#include "util/StringUtil.hxx"
+#include "util/TruncateString.hxx"
+#include "util/TimeConvert.hxx"
 
 #include <algorithm>
 
@@ -72,7 +74,7 @@ SanitizeString(const char *s, char *buffer, size_t buffer_size) noexcept
 	if (!HasUnsafeChar(s))
 		return s;
 
-	char *end = CopyString(buffer, s, buffer_size);
+	char *end = CopyTruncateString(buffer, s, buffer_size);
 	std::replace_if(buffer, end, IsUnsafeChar, ' ');
 	return buffer;
 }
@@ -85,15 +87,13 @@ TagGetter(const void *object, const char *name) noexcept
 	auto &ctx = const_cast<FormatTagContext &>(_ctx);
 
 	if (strcmp(name, "iso8601") == 0) {
-		time_t t = time(nullptr);
-#ifdef _WIN32
-		const struct tm *tm2 = gmtime(&t);
-#else
 		struct tm tm;
-		const struct tm *tm2 = gmtime_r(&t, &tm);
-#endif
-		if (tm2 == nullptr)
+
+		try {
+			tm = GmTime(std::chrono::system_clock::now());
+		} catch (...) {
 			return "";
+		}
 
 		strftime(ctx.buffer, sizeof(ctx.buffer),
 #ifdef _WIN32
@@ -105,7 +105,7 @@ TagGetter(const void *object, const char *name) noexcept
 #else
 			 "%FT%TZ",
 #endif
-			 tm2);
+			 &tm);
 		return ctx.buffer;
 	}
 
