@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,60 +19,29 @@
 
 #include "config.h"
 #include "Tag.hxx"
-#include "TagPool.hxx"
-#include "TagBuilder.hxx"
-#include "util/ASCII.hxx"
+#include "Pool.hxx"
+#include "Builder.hxx"
 
 #include <assert.h>
-#include <string.h>
-
-TagType
-tag_name_parse(const char *name) noexcept
-{
-	assert(name != nullptr);
-
-	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; ++i) {
-		assert(tag_item_names[i] != nullptr);
-
-		if (strcmp(name, tag_item_names[i]) == 0)
-			return (TagType)i;
-	}
-
-	return TAG_NUM_OF_ITEM_TYPES;
-}
-
-TagType
-tag_name_parse_i(const char *name) noexcept
-{
-	assert(name != nullptr);
-
-	for (unsigned i = 0; i < TAG_NUM_OF_ITEM_TYPES; ++i) {
-		assert(tag_item_names[i] != nullptr);
-
-		if (StringEqualsCaseASCII(name, tag_item_names[i]))
-			return (TagType)i;
-	}
-
-	return TAG_NUM_OF_ITEM_TYPES;
-}
 
 void
-Tag::Clear()
+Tag::Clear() noexcept
 {
 	duration = SignedSongTime::Negative();
 	has_playlist = false;
 
-	tag_pool_lock.lock();
-	for (unsigned i = 0; i < num_items; ++i)
-		tag_pool_put_item(items[i]);
-	tag_pool_lock.unlock();
+	{
+		const std::lock_guard<Mutex> protect(tag_pool_lock);
+		for (unsigned i = 0; i < num_items; ++i)
+			tag_pool_put_item(items[i]);
+	}
 
 	delete[] items;
 	items = nullptr;
 	num_items = 0;
 }
 
-Tag::Tag(const Tag &other)
+Tag::Tag(const Tag &other) noexcept
 	:duration(other.duration), has_playlist(other.has_playlist),
 	 num_items(other.num_items),
 	 items(nullptr)
@@ -80,23 +49,22 @@ Tag::Tag(const Tag &other)
 	if (num_items > 0) {
 		items = new TagItem *[num_items];
 
-		tag_pool_lock.lock();
+		const std::lock_guard<Mutex> protect(tag_pool_lock);
 		for (unsigned i = 0; i < num_items; i++)
 			items[i] = tag_pool_dup_item(other.items[i]);
-		tag_pool_lock.unlock();
 	}
 }
 
-Tag *
-Tag::Merge(const Tag &base, const Tag &add)
+std::unique_ptr<Tag>
+Tag::Merge(const Tag &base, const Tag &add) noexcept
 {
 	TagBuilder builder(add);
 	builder.Complement(base);
 	return builder.CommitNew();
 }
 
-Tag *
-Tag::MergeReplace(Tag *base, Tag *add)
+std::unique_ptr<Tag>
+Tag::Merge(std::unique_ptr<Tag> base, std::unique_ptr<Tag> add) noexcept
 {
 	if (add == nullptr)
 		return base;
@@ -104,11 +72,7 @@ Tag::MergeReplace(Tag *base, Tag *add)
 	if (base == nullptr)
 		return add;
 
-	Tag *tag = Merge(*base, *add);
-	delete base;
-	delete add;
-
-	return tag;
+	return Merge(*base, *add);
 }
 
 const char *
@@ -128,3 +92,68 @@ Tag::HasType(TagType type) const noexcept
 {
 	return GetValue(type) != nullptr;
 }
+
+static TagType
+DecaySort(TagType type) noexcept
+{
+	switch (type) {
+	case TAG_ARTIST_SORT:
+		return TAG_ARTIST;
+
+	case TAG_ALBUM_SORT:
+		return TAG_ALBUM;
+
+	case TAG_ALBUM_ARTIST_SORT:
+		return TAG_ALBUM_ARTIST;
+
+	default:
+		return TAG_NUM_OF_ITEM_TYPES;
+	}
+}
+
+static TagType
+Fallback(TagType type) noexcept
+{
+	switch (type) {
+	case TAG_ALBUM_ARTIST:
+		return TAG_ARTIST;
+
+	case TAG_MUSICBRAINZ_ALBUMARTISTID:
+		return TAG_MUSICBRAINZ_ARTISTID;
+
+	default:
+		return TAG_NUM_OF_ITEM_TYPES;
+	}
+}
+
+const char *
+Tag::GetSortValue(TagType type) const noexcept
+{
+	const char *value = GetValue(type);
+	if (value != nullptr)
+		return value;
+
+	/* try without *_SORT */
+	const auto no_sort_type = DecaySort(type);
+	if (no_sort_type != TAG_NUM_OF_ITEM_TYPES) {
+		value = GetValue(no_sort_type);
+		if (value != nullptr)
+			return value;
+	}
+
+	/* fall back from TAG_ALBUM_ARTIST to TAG_ALBUM */
+
+	type = Fallback(type);
+	if (type != TAG_NUM_OF_ITEM_TYPES)
+		return GetSortValue(type);
+
+	if (no_sort_type != TAG_NUM_OF_ITEM_TYPES) {
+		type = Fallback(no_sort_type);
+		if (type != TAG_NUM_OF_ITEM_TYPES)
+			return GetSortValue(type);
+	}
+
+	/* finally fall back to empty string */
+
+	return "";
+}
