@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 #include "pcm/PcmConvert.hxx"
 #include "util/ConstBuffer.hxx"
 #include "util/StaticFifoBuffer.hxx"
-#include "Log.hxx"
+#include "util/PrintException.hxx"
 
 #include <assert.h>
 #include <stddef.h>
@@ -62,7 +62,7 @@ try {
 	while (true) {
 		{
 			const auto dest = buffer.Write();
-			assert(!dest.IsEmpty());
+			assert(!dest.empty());
 
 			ssize_t nbytes = read(0, dest.data, dest.size);
 			if (nbytes <= 0)
@@ -72,10 +72,10 @@ try {
 		}
 
 		auto src = buffer.Read();
-		assert(!src.IsEmpty());
+		assert(!src.empty());
 
 		src.size -= src.size % in_frame_size;
-		if (src.IsEmpty())
+		if (src.empty())
 			continue;
 
 		buffer.Consume(src.size);
@@ -86,10 +86,19 @@ try {
 						   output.size);
 	}
 
+	while (true) {
+		auto output = state.Flush();
+		if (output.IsNull())
+			break;
+
+		gcc_unused ssize_t ignored = write(1, output.data,
+						   output.size);
+	}
+
 	state.Close();
 
 	return EXIT_SUCCESS;
-} catch (const std::exception &e) {
-	LogError(e);
+} catch (...) {
+	PrintException(std::current_exception());
 	return EXIT_FAILURE;
 }
